Package: pboost
Title: Profile Boosting Framework for Parametric Models
Version: 0.4.11
Maintainer: Zengchao Xu <zengc.xu@aliyun.com>
Authors@R: 
    person("Zengchao", "Xu", , "zengc.xu@aliyun.com", role = c("aut", "cre", "cph"))
Description: 
    A profile boosting framework for feature selection in parametric models.
    It offers a unified interface pboost() and several wrapped models, including linear model, generalized linear models, quantile regression, Cox proportional hazards model, beta regression, spatial auto-regressive models.
Imports: stats, Matrix, MASS, betareg, quantreg, survival, Formula (>=
        1.2.5), glmnet, ncvreg, rqPen, spatialreg
License: GPL (>= 3)
Encoding: UTF-8
URL: https://github.com/paradoxical-rhapsody/pboost
BugReports: https://github.com/paradoxical-rhapsody/pboost/issues
Depends: R (>= 4.1.0)
RoxygenNote: 8.0.0
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-24 06:41:58 UTC; zengc
Author: Zengchao Xu [aut, cre, cph]
Repository: CRAN
Date/Publication: 2026-05-24 07:00:02 UTC
Built: R 4.6.0; ; 2026-05-24 09:33:10 UTC; unix

Package: pboost
Title: Profile Boosting Framework for Parametric Models
Version: 0.4.11
Maintainer: Zengchao Xu <zengc.xu@aliyun.com>
Authors@R: 
    person("Zengchao", "Xu", , "zengc.xu@aliyun.com", role = c("aut", "cre", "cph"))
Description: 
    A profile boosting framework for feature selection in parametric models.
    It offers a unified interface pboost() and several wrapped models, including linear model, generalized linear models, quantile regression, Cox proportional hazards model, beta regression, spatial auto-regressive models.
Imports: stats, Matrix, MASS, betareg, quantreg, survival, Formula (>=
        1.2.5), glmnet, ncvreg, rqPen, spatialreg
License: GPL (>= 3)
Encoding: UTF-8
URL: https://github.com/paradoxical-rhapsody/pboost
BugReports: https://github.com/paradoxical-rhapsody/pboost/issues
Depends: R (>= 4.1.0)
RoxygenNote: 8.0.0
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-24 06:41:58 UTC; zengc
Author: Zengchao Xu [aut, cre, cph]
Repository: CRAN
Date/Publication: 2026-05-24 07:00:02 UTC
Built: R 4.5.2; ; 2026-05-24 09:17:20 UTC; unix

# Generated by roxygen2: do not edit by hand

S3method(BIC,sarpboost)
S3method(EBIC,default)
S3method(coef,penaltyglmnet)
S3method(coef,penaltyncvreg)
S3method(coef,penaltyrq)
S3method(coef,sarpboost)
S3method(logLik,sarpboost)
S3method(residuals,sarpboost)
export(EBIC)
export(fbetareg)
export(fcoxph)
export(fglm)
export(fglm.fit)
export(flagsarlm)
export(flm)
export(flm.fit)
export(frq)
export(frq.fit)
export(frs)
export(fsar.fit)
export(get_rho)
export(pbetareg)
export(pboost)
export(pcoxph)
export(pen_glmnet)
export(pen_ncvreg)
export(pen_rq)
export(pen_sar)
export(pggm)
export(pglm)
export(pglm.fit)
export(plagsarlm)
export(plm)
export(plm.fit)
export(prq)
export(prq.fit)
export(psar.fit)
export(sar.fit)
export(set_rook_matrix)
import(Formula)
import(Matrix)
importFrom(MASS,ginv)
importFrom(betareg,betareg)
importFrom(betareg,betareg.control)
importFrom(glmnet,cv.glmnet)
importFrom(glmnet,glmnet)
importFrom(ncvreg,cv.ncvreg)
importFrom(ncvreg,cv.ncvsurv)
importFrom(quantreg,rq)
importFrom(quantreg,rq.fit)
importFrom(quantreg,rq.wfit)
importFrom(rqPen,qic.select)
importFrom(rqPen,rq.pen)
importFrom(spatialreg,lagsarlm)
importFrom(stats,AIC)
importFrom(stats,BIC)
importFrom(stats,as.formula)
importFrom(stats,coef)
importFrom(stats,fitted)
importFrom(stats,formula)
importFrom(stats,gaussian)
importFrom(stats,glm)
importFrom(stats,glm.fit)
importFrom(stats,lm)
importFrom(stats,lm.fit)
importFrom(stats,logLik)
importFrom(stats,model.frame)
importFrom(stats,model.response)
importFrom(stats,na.omit)
importFrom(stats,nobs)
importFrom(stats,optimize)
importFrom(stats,predict)
importFrom(stats,qlogis)
importFrom(stats,residuals)
importFrom(stats,terms)
importFrom(survival,coxph)
importFrom(survival,coxph.fit)

#' @rdname penalization
#' 
#' @description
#' - `pen_glmnet(x, y, family)`: LASSO penalized models.
#' 
#' @noRd
pen_glmnet <- function(x, y, family) {
    stopifnot( family %in% c("gaussian", "binomial", "cox") )

    suppressWarnings(
        egg <- cv.glmnet(
            x = x,
            y = y,
            family = family,
            standardize = FALSE,
            intercept = FALSE,
            type.measure = "deviance",
            nfolds = 10
        )
    )

    class(egg) <- c("penaltyglmnet", class(egg))
    return(egg)
}



#' @title Extract Coefficients from Lasso Penalized Models
#'
#' @param object Object.
#' @param ... Additional arguments (not used).
#' 
#' @return Named vector of non-zero coefficients of under cross-validation error.
#' 
#' @noRd
coef.penaltyglmnet <- function(object, ...) {
    idx <- object[["index"]]['min', 'Lambda']
    b0 <- object[["glmnet.fit"]][["beta"]][, idx, drop=TRUE]
    return(b0[abs(b0) >= 1e-4])
}

#' @name fglm
#' @title Forward Regression Selection for Generalized Linear Models
#' 
#' @description `fglm()` inherits the usage of [glm], and performs forward regression selection for generalized linear models.
#' 
#' @param formula Parameter passed to [glm].
#' @param data Parameter passed to [glm].
#' @param family Parameter passed to [glm].
#' @param weights Parameter passed to [glm].
#' @param subset Parameter passed to [glm].
#' @param na.action Parameter passed to [glm].
#' @param start Parameter passed to [glm].
#' @param etastart Parameter passed to [glm].
#' @param mustart Parameter passed to [glm].
#' @param offset Parameter passed to [glm].
#' @param control Parameter passed to [glm].
#' @param model Parameter passed to [glm].
#' @param method Parameter passed to [glm].
#' @param x Parameter passed to [glm].
#' @param y Parameter passed to [glm].
#' @param singular.ok Parameter passed to [glm].
#' @param contrasts Parameter passed to [glm].
#' @param ... Parameters passed to [glm].
#' @param intercept Parameter passed to [glm.fit].
#' 
#' @param selectFun Parameter passed to [pboost::frs].
#' @param stopFun Parameter passed to [pboost::frs].
#' @param keep Parameter passed to [pboost::frs].
#' @param maxK Parameter passed to [pboost::frs].
#' @param verbose Parameter passed to [pboost::frs].
#' 
#' @return A `glm` model object fitted on the selected features.
#' 
NULL
#> NULL



#' @rdname fglm
#' @order 1
#' @export
fglm <- function(formula, family = gaussian, data, weights, subset,
                na.action, start = NULL, etastart, mustart, offset,
                control = list(...), model = TRUE, method = "glm.fit",
                x = FALSE, y = TRUE, singular.ok = TRUE, contrasts = NULL, ...,
                selectFun = logLik, stopFun = "EBIC",
                keep = NULL, maxK = NULL, verbose = FALSE) {

    fml <- Formula(formula)
    mf <- model.frame(fml, data=data)

    yvec <- model.part(fml, data=mf, lhs=1, drop=TRUE)
    xmat <- model.part(fml, data=mf, rhs=1) |> as.matrix()
    use.intercept <- attr(terms(mf), "intercept") == 1L


    mc <- match.call(expand.dots = TRUE)
    provided_args <- as.list(mc)[-1]
    provided_args$formula <- NULL
    provided_args$data <- NULL

    args <- list(
        fitFun = glm,
        yvec = yvec,
        xmat = xmat,
        use.intercept = use.intercept
    )
    args <- c(args, provided_args)

    return(do.call(frs, args))
}



#' @rdname fglm
#' @order 2
#' @export
fglm.fit <- function(x, y, weights = rep.int(1, NROW(y)), start = NULL, etastart = NULL,
                      mustart = NULL, offset = rep.int(0, NROW(y)), family = gaussian(),
                      control = list(), intercept = TRUE, singular.ok = TRUE,
                      selectFun = "logLik", stopFun = "EBIC",
                      keep = NULL, maxK = NULL, verbose = FALSE) {

    n <- NROW(x)
    p <- NCOL(x)

    if (is.character(selectFun) && selectFun == "logLik")
        selectFun <- function(object) {
            class(object) <- "glm"
            return(logLik(object))
        }
    
    if (is.character(stopFun) && stopFun == "EBIC")
        stopFun <- function(object) {
            class(object) <- "glm"

            ebic.r <- max( 0.0, 1.0 - log(n) / (2.0 * log(p)) )
            stopifnot( ebic.r >= 0 )

            dof <- length(coef(object))
            ebic.penalty <- 2.0 * ebic.r * lchoose(p - length(keep), dof - length(keep))
            stopifnot( is.finite(ebic.penalty) )

            return(BIC(object) + ebic.penalty)
        }

    
    mc <- match.call(expand.dots = TRUE)
    provided_args <- as.list(mc)[-1]
    provided_args <- provided_args[!(names(provided_args) %in% c("selectFun", "stopFun"))]
    provided_args$x <- NULL
    provided_args$y <- NULL

    args <- list(
        fitFun = glm.fit,
        yvec = y,
        xmat = x,
        selectFun = selectFun,
        stopFun = stopFun,
        use.formula = FALSE
    )
    args <- c(args, provided_args)

    return(do.call(frs, args))
}

#' @name psar
#' 
#' @title Profile Boosting for Spatial Auto-regressive Model
#' 
#' @description Profile boosting variable selection for spatial auto-regressive (SAR) model
#' \deqn{\bm{y} = \rho W \bm{y} + X \beta + \bm{\varepsilon},}
#' where \eqn{\bm{y}, \bm{\varepsilon} \in \mathbb{R}^n}, \eqn{X \in \mathbb{R}^{n \times p}},
#' \eqn{\rho \in (-1, 1)}.
#' 
#' @param x Numeric feature matrix.
#' @param y Response vector.
#' @param w Weight matrix (row-sum scaled being one).
#' @param formula Parameters passed to [spatialreg::lagsarlm].
#' @param data Parameters passed to [spatialreg::lagsarlm].
#' @param listw Parameters passed to [spatialreg::lagsarlm].
#' @param na.action Parameters passed to [spatialreg::lagsarlm].
#' @param Durbin Parameters passed to [spatialreg::lagsarlm].
#' @param type Parameters passed to [spatialreg::lagsarlm].
#' @param method Parameters passed to [spatialreg::lagsarlm].
#' @param quiet Parameters passed to [spatialreg::lagsarlm].
#' @param zero.policy Parameters passed to [spatialreg::lagsarlm].
#' @param interval Parameters passed to [spatialreg::lagsarlm].
#' @param tol.solve Parameters passed to [spatialreg::lagsarlm].
#' @param trs Parameters passed to [spatialreg::lagsarlm].
#' @param control Parameters passed to [spatialreg::lagsarlm].
#' 
#' @param stopFun Parameter passed to [pboost::pboost].
#' @param keep Parameter passed to [pboost::pboost].
#' @param maxK Parameter passed to [pboost::pboost].
#' @param verbose Parameter passed to [pboost::pboost].
#' 
#' @return Model object fitted on the selected features.
#' 
#' @examples
#' set.seed(2026)
#' 
#' w <- set_rook_matrix(15, 15)
#' 
#' n <- NROW(w)
#' p <- 30
#' x <- matrix(rnorm(n*p), n) %*% chol(0.7^abs(outer(1:p, 1:p, "-")))
#' eta <- drop(x[, 1:3] %*% runif(3, 1.5, 2.0))
#' 
#' rho0 <- 0.2
#' sig0 <- 1.0
#' y <- solve(diag(n) - rho0 * w, rnorm(n, eta, sd=sig0)) |> drop()
#' 
#' ## ---------- pboost ----------
#' system.time( egg <- psar.fit(x, y, w, verbose=TRUE) )
#' y.tilde <- (diag(NROW(x)) - egg[["rho"]] * w) %*% y
#' 
#' beta.hat <- egg[["beta"]]
#' idx <- as.integer(sub("[[:alpha:]]", "", names(beta.hat)))
#' sig2.hat <- mean( (y.tilde - drop(x[, idx, drop=FALSE] %*% beta.hat))^2 )
#' print( egg[["sig2"]] - sig2.hat )
#' 
#' \dontrun{
#' system.time(
#' plagsarlm(y ~ ., data=data.frame(y, x), listw=spdep::mat2listw(w, style="W"), verbose=TRUE)
#' )
#' }
#' 
#' ## ---------- frs ----------
#' fsar.fit(x, y, w, verbose=TRUE)
#' 
#' \dontrun{
#' system.time(
#' flagsarlm(y ~ ., data=data.frame(y, x), listw=spdep::mat2listw(w, style="W"), verbose=TRUE)
#' )
#' }
#' 
NULL
#> NULL



#' @rdname psar
#' @order 1
#' @export
plagsarlm <- function(formula, data = list(), listw, na.action, Durbin, type,
                    method = "eigen", quiet = NULL, zero.policy = NULL, interval = NULL,
                    tol.solve = .Machine$double.eps, trs = NULL, control = list(),
                    stopFun = "EBIC",
                    keep = NULL, maxK = NULL, verbose = FALSE) {

    fml <- Formula(formula)
    mf <- model.frame(fml, data=data)

    yvec <- model.part(fml, data=mf, lhs=1, drop=TRUE)
    xmat <- model.part(fml, data=mf, rhs=1) |> as.matrix()
    use.intercept <- attr(terms(mf), "intercept") == 1L


    mc <- match.call(expand.dots = TRUE)
    provided_args <- as.list(mc)[-1]
    provided_args <- provided_args[!(names(provided_args) %in% c("formula", "data"))]

    fitFun <- function(...) {
        dots <- list(...)
        return(do.call(lagsarlm, dots))
    }

    args <- list(
        fitFun = fitFun,
        yvec = yvec,
        xmat = xmat,
        use.intercept = use.intercept,
        scoreFun = residuals
    )
    args <- c(args, provided_args)

    return(do.call(pboost, args))
}



#' @rdname psar
#' @order 2
#' @export
psar.fit <- function(x, y, w,
                stopFun = "EBIC",
                keep = NULL, maxK = NULL, verbose = FALSE) {
    stopifnot( is.matrix(x) )
    stopifnot( all( abs(rowSums(w) - 1.0) <= 1e-6 ) )
    stopifnot( all(w >= 0.0) )
    stopifnot( !anyNA(x) )
    stopifnot( !anyNA(y) )
    stopifnot( !anyNA(w) )

    n <- NROW(x)
    p <- NCOL(x)

    if (is.character(stopFun) && stopFun == "EBIC")
        stopFun <- function(object) {
            ebic.r <- max( 0.0, 1.0 - log(n) / (2.0 * log(p)) )
            stopifnot( ebic.r >= 0 )

            dof <- attr(object, "dof")
            ebic.penalty <- 2.0 * ebic.r * lchoose(p - length(keep), dof - length(keep))
            stopifnot( is.finite(ebic.penalty) )
            stopifnot( length(ebic.penalty) == 1 )

            return(BIC(object) + ebic.penalty)
        }

    
    mc <- match.call(expand.dots = TRUE)
    provided_args <- as.list(mc)[-1]
    provided_args <- provided_args[!(names(provided_args) %in% c("x", "y", "scoreFun", "stopFun"))]

    args <- list(
        fitFun = sar.fit,
        yvec = y,
        xmat = x,
        scoreFun = residuals,
        stopFun = stopFun,
        use.formula = FALSE
    )
    args <- c(args, provided_args)

    return(do.call(pboost, args))
}

#' @rdname penalization
#' 
#' @description
#' - `pen_sar(x, y, rho, w, penalty)`: penalized spatial auto-regressive models.
#' 
#' @noRd
pen_sar <- function(x, y, rho, w, penalty) {
    stopifnot( penalty %in% c("MCP", "SCAD", "lasso") )

    rho <- ifelse(missing(rho) || is.null(rho), get_rho(x, y, w), rho)
    stopifnot( length(rho) == 1 & is.numeric(rho) )

    n <- length(y)
    A.rho <- diag(n) - rho * w
    y <- drop(A.rho %*% y)
    stopifnot( length(y) == n )

    if (penalty == "lasso")
        egg <- pen_glmnet(x, y, "gaussian")
    else
        egg <- pen_ncvreg(x, y, "gaussian", penalty)

    class(egg) <- c("penltysar", class(egg))
    return(egg)
}

#' @title Forward Stepwise Spatial Auto-Regressive Model
#' 
#' @description Forward stepwise strategy for spatial auto-regressive model.
#' 
#' @param x Matrix of covariates.
#' @param y Vector of response.
#' @param w Weight matrix (row-sum scaled being one).
#' 
#' @param selectFun Parameter passed to [pboost::frs].
#' @param stopFun Parameter passed to [pboost::frs].
#' @param keep Parameter passed to [pboost::frs].
#' @param maxK Parameter passed to [pboost::frs].
#' @param verbose Parameter passed to [pboost::frs].
#' 
#' @return Model object fitted on the selected features.
#' 
#' @export
fsar <- function(x, y, w,
                selectFun = logLik,
                stopFun = "EBIC",
                keep = NULL, maxK = NULL, verbose = FALSE) {
    stopifnot( is.matrix(x) )
    stopifnot( all( abs(rowSums(w) - 1.0) <= 1e-12 ) )
    stopifnot( all(w >= 0.0) )
    stopifnot( !anyNA(x) )
    stopifnot( !anyNA(y) )
    stopifnot( !anyNA(w) )

    n <- NROW(x)
    p <- NCOL(x)

    stopFun <- function(object) {
        ebic.r <- max( 0.0, 1.0 - log(n) / (2.0 * log(p)) )
        stopifnot( ebic.r >= 0 )

        dof <- attr(object, "dof")
        ebic.penalty <- 2.0 * ebic.r * lchoose(p - length(keep), dof - length(keep))
        stopifnot( is.finite(ebic.penalty) )
        stopifnot( length(ebic.penalty) == 1 )

        return(BIC(object) + ebic.penalty)
    }

    egg <- frs(
        yvec = y,
        xmat = x,
        fitFun = sar.fit,
        w = w,
        use.formula = FALSE,
        selectFun = selectFun,
        stopFun = stopFun,
        keep = keep,
        maxK = maxK,
        verbose = verbose
    )
    class(egg) <- c("psar", class(egg))
    return(egg)
}

#' @name frq
#' @title Forward Regression Selection for Quantile Regression Models
#' 
#' @description
#' `frq()` inherits the usage of the function [quantreg::rq], and performs forward regression selection for quantile regression models.
#' 
#' @param formula Parameter passed to [quantreg::rq].
#' @param tau Parameter passed to [quantreg::rq].
#' @param data Parameter passed to [quantreg::rq].
#' @param x Parameter passed to [quantreg::rq.fit].
#' @param y Parameter passed to [quantreg::rq.fit].
#' @param subset Parameter passed to [quantreg::rq].
#' @param weights Parameter passed to [quantreg::rq].
#' @param na.action Parameter passed to [quantreg::rq].
#' @param method Parameter passed to [quantreg::rq].
#' @param model Parameter passed to [quantreg::rq].
#' @param contrasts Parameter passed to [quantreg::rq].
#' @param ... Parameters passed to [quantreg::rq].
#' 
#' @param selectFun Parameter passed to [pboost::frs].
#' @param stopFun Parameter passed to [pboost::frs].
#' @param keep Parameter passed to [pboost::frs].
#' @param maxK Parameter passed to [pboost::frs].
#' @param verbose Parameter passed to [pboost::frs].
#' 
#' @return A `rq` model object fitted on the selected features.
#' 
#' NULL
NULL
#> NULL



#' @rdname frq
#' @order 1
#' @export 
frq <- function(formula, tau = 0.5, data, subset, weights, na.action,
                method = "br", model = TRUE, contrasts = NULL, ...,
                selectFun = logLik, stopFun = "EBIC",
                keep = NULL, maxK = NULL, verbose = FALSE) {

    stopifnot( length(tau) == 1 )

    fml <- Formula(formula)
    mf <- model.frame(fml, data=data)

    yvec <- model.part(fml, data=mf, lhs=1, drop=TRUE)
    xmat <- model.part(fml, data=mf, rhs=1) |> as.matrix()
    use.intercept <- attr(terms(mf), "intercept") == 1L


    n <- NROW(xmat)
    p <- NCOL(xmat)
    if (is.character(stopFun) && stopFun == "EBIC")
        stopFun <- function(object) {
            ebic.r <- max( 0.0, 1.0 - log(n) / (2.0 * log(p)) )
            stopifnot( ebic.r >= 0 )

            dof <- attr(logLik(object), "df")
            ebic.penalty <- 2.0 * ebic.r * lchoose(p - length(keep), dof - length(keep))
            stopifnot( is.finite(ebic.penalty) )

            # `BIC` for class `rq` is equivalent to `AIC` with negative `k`
            # return( -2*as.numeric(obj.loglik) + dof * log(n) + ebic.penalty )
            return( AIC(object, k=-1) + ebic.penalty )
        }

    mc <- match.call(expand.dots = TRUE)
    provided_args <- as.list(mc)[-1]
    provided_args <- provided_args[!(names(provided_args) %in% c("formula", "data", "stopFun"))]

    args <- list(
        fitFun = rq,
        yvec = yvec,
        xmat = xmat,
        use.intercept = use.intercept,
        stopFun = stopFun
    )
    args <- c(args, provided_args)

    return(do.call(frs, args))
}




#' @rdname frq
#' @order 2
#' @export 
frq.fit <- function(x, y, tau = 0.5, method = "br", ...,
                    selectFun = "logLik", stopFun = "EBIC",
                    keep = NULL, maxK = NULL, verbose = FALSE) {

    n <- NROW(x)
    p <- NCOL(x)

    if (is.character(selectFun) && selectFun == "logLik")
        selectFun <- function(object) {
            Rho <- function(u, tau) u * (tau - (u < 0))
            fid <- sum(Rho(object$residuals, tau))
            loglik <- n * (log(tau * (1 - tau)) - 1 - log(fid/n))
        }

    if (is.character(stopFun) && stopFun == "EBIC")
        stopFun <- function(object) {
            ebic.r <- max( 0.0, 1.0 - log(n) / (2.0 * log(p)) )
            stopifnot( ebic.r >= 0 )

            dof <- length(object$coefficients)
            ebic.penalty <- 2.0 * ebic.r * lchoose(p - length(keep), dof - length(keep))
            stopifnot( is.finite(ebic.penalty) )

            Rho <- function(u, tau) u * (tau - (u < 0))
            fid <- sum(Rho(object$residuals, tau))
            loglik <- n * (log(tau * (1 - tau)) - 1 - log(fid/n))

            return( -2*loglik + dof * log(n) + ebic.penalty )
        }

    mc <- match.call(expand.dots = TRUE)
    provided_args <- as.list(mc)[-1]
    provided_args <- provided_args[!(names(provided_args) %in% c("x", "y", "selectFun", "stopFun"))]

    weights <- provided_args$weights
    fitFun <- ifelse(!is.null(weights) && length(weights), rq.wfit, rq.fit)

    args <- list(
        fitFun = fitFun,
        yvec = y,
        xmat = x,
        selectFun = selectFun,
        stopFun = stopFun,
        use.formula = FALSE
    )
    args <- c(args, provided_args)

    return(do.call(frs, args))
}

#' @name fbetareg
#' @title Forward Regression Selection for Beta Regression Models
#' 
#' @description
#' `fbetareg()` inherits the usage of the function [betareg::betareg], and performs forward regression selection for beta regression models.
#'
#' @param formula Parameter passed to [betareg::betareg].
#' @param data Parameter passed to [betareg::betareg].
#' @param subset Parameter passed to [betareg::betareg].
#' @param na.action Parameter passed to [betareg::betareg].
#' @param weights Parameter passed to [betareg::betareg].
#' @param offset Parameter passed to [betareg::betareg].
#' @param link Parameter passed to [betareg::betareg].
#' @param link.phi Parameter passed to [betareg::betareg].
#' @param type Parameter passed to [betareg::betareg].
#' @param dist Parameter passed to [betareg::betareg].
#' @param nu Parameter passed to [betareg::betareg].
#' @param control Parameter passed to [betareg::betareg].
#' @param model Parameter passed to [betareg::betareg].
#' @param y Parameter passed to [betareg::betareg].
#' @param x Parameter passed to [betareg::betareg].
#' @param ... Parameters passed to [betareg::betareg].
#' 
#' @param selectFun Parameter passed to [pboost::frs].
#' @param stopFun Parameter passed to [pboost::frs].
#' @param keep Parameter passed to [pboost::frs].
#' @param maxK Parameter passed to [pboost::frs].
#' @param verbose Parameter passed to [pboost::frs].
#' 
#' @return A `betareg` model object fitted on the selected features.
#' 
#' @export
fbetareg <- function(formula, data, subset, na.action, weights, offset,
                    link = c("logit", "probit", "cloglog", "cauchit", "log","loglog"),
                    link.phi = NULL, type = c("ML", "BC", "BR"),
                    dist = NULL, nu = NULL, control = betareg.control(...),
                    model = TRUE, y = TRUE, x = FALSE, ...,
                    selectFun = logLik, stopFun = "EBIC",
                    keep = NULL, maxK = NULL, verbose = FALSE) {

    fml <- Formula(formula)
    mf <- model.frame(fml, data=data)

    yvec <- model.part(fml, data=mf, lhs=1, drop=TRUE)
    xmat <- model.part(fml, data=mf, rhs=1) |> as.matrix()
    use.intercept <- attr(terms(mf), "intercept") == 1L


    mc <- match.call(expand.dots = TRUE)
    provided_args <- as.list(mc)[-1]
    provided_args$formula <- NULL
    provided_args$data <- NULL

    args <- list(
        fitFun = betareg,
        yvec = yvec,
        xmat = xmat,
        use.intercept = use.intercept
    )
    args <- c(args, provided_args)

    return(do.call(frs, args))
}

#' @name pboost
#' @title Profile Boosting Framework
#' 
#' @description
#' `pboost()` is the generic workhorse function of profile boosting
#' framework for parametric regression.
#' 
#' @param yvec Response vector.
#' @param xmat Numeric feature matrix.
#' @param fitFun Function to fit the empirical risk function in
#'    the form `fitFun(formula, data, ...)`.
#' @param scoreFun Function to compute the derivative, denoted by \eqn{\frac{\partial \ell(y, \eta)}{\partial \eta}}, of empirical risk function in the form `scoreFun(object)`, where `object` is returned by `fitFun`.
#' `scoreFun()` should return a vector with the same length of `y` in `data`.
#' @param stopFun Stopping rule for profile boosting, which has the form
#'    `stopFun(object)` to evaluate the performance of model `object` returned
#'    by `fitFun`, such as [EBIC] or [BIC].
#' @param ... Additional arguments to be passed to `fitFun`.
#' @param use.formula Whether to use formula interface for model fitting. Default to `TRUE`.
#' When `use.formula=TRUE`, the the model fitting function has the form `fitFun(formula, data, ...)`; otherwise, `fitFun(x, y, ...)`.
#' @param use.intercept Include intercept in the model fitting? Valid only when `use.formula=TRUE`.
#' @param maxK Maximal number of identified features.
#'    If `maxK` is specified, it will suppress `stopFun`, saying that the
#'    profile boosting continues until the procedure identifies `maxK` features.
#'    The pre-specified features in `keep` are counted toward `maxK`.
#' @param keep Vector of indices or feature names initial features to include.
#' @param verbose Print the procedure path?
#' 
#' @return Model object fitted on the selected features.
#' 
#' @seealso [pboost::pbetareg], [pboost::pcoxph], [pboost::pglm], [pboost::plm], [pboost::prq], [pboost::psar].
#' 
NULL




#' @rdname pboost
#' @order 1
#' @export
pboost <- function(yvec, xmat, fitFun, scoreFun, stopFun = "EBIC", ...,
                    use.formula = TRUE,
                    use.intercept = TRUE,
                    keep = NULL, maxK = NULL, verbose = FALSE) {
    stopifnot( NROW(yvec) == NROW(xmat) )
    stopifnot( is.matrix(xmat), is.numeric(xmat) )

    n <- NROW(xmat)
    p <- NCOL(xmat)

    xnames <- colnames(xmat)
    if (is.null(xnames)) {
        xnames <- paste0("x", seq_len(p))
        colnames(xmat) <- xnames
    }
    stopifnot( length(xnames) == p )

    if ( is.character(stopFun) && use.formula && stopFun == "EBIC")
        stopFun <- function(object){
            ebic.r <- max( 0.0, 1.0 - log(n) / (2.0 * log(p)) )
            stopifnot( ebic.r >= 0 )

            dof <- length(all.vars(formula(object)[[3]]))
            ebic.penalty <- 2.0 * ebic.r * lchoose(p - length(keep), dof - length(keep))
            stopifnot( is.finite(ebic.penalty) )

            # return(-2*logLik(object) + dof * log(n) + ebic.penalty)
            return(BIC(object) + ebic.penalty)
        }

    stopifnot( is.character(keep) || is.numeric(keep) || is.null(keep) )
    if (!is.null(keep)) {
        if (is.character(keep)) {
            stopifnot( all(keep %in% xnames) )
        } else {
            stopifnot( all(keep %in% seq_len(p)) )
            keep <- xnames[keep]
        }
    }
    stopifnot( is.character(keep) || is.null(keep) )

    if (!is.null(maxK))
        maxK <- min( maxK, p, n-1 )

    showiter <- function(verbose, x.star, level) {
        if (verbose)
            if (missing(x.star))
                message(sprintf("Initial model: level=%.3f", level))
            else 
                message(sprintf("Adding %s: level=%.3f", x.star, level))
    }

    fit_formula <- function(fml) {
        if (use.formula) {
            return(fitFun(
                formula = fml,
                data = data.frame(yvec, xmat[, all.vars(fml[[3]]), drop=FALSE]),
                ...
            ))
        } else {
            return(fitFun(x = xmat[, all.vars(fml[[3]]), drop=FALSE], y = yvec, ...))
        }
    }

    lhs <- "yvec"
    rhs <- paste(c(as.integer(use.intercept), keep), collapse=" + ")
    fml <- as.formula(paste(c(lhs, rhs), collapse=" ~ "))
    egg <- fit_formula(fml)
    level <- stopFun(egg)
    showiter(verbose, level=level)

    if ( !is.null(maxK) && !is.null(keep) && (length(keep) >= maxK) ) {
        warning("'length(keep)' is not less than 'maxK', thus fitting on 'keep'.")
        return(egg)
    }

    while (TRUE) {
        candidates <- setdiff(xnames, all.vars(fml[[3]]))
        if (length(candidates) == 0)
            break

        x.star <- crossprod(xmat[, candidates, drop=FALSE], scoreFun(egg)) |>
            drop() |> abs() |> which.max() |> names()
        stopifnot( length(x.star) == 1 )
        stopifnot( !(x.star %in% all.vars(fml[[3]])) )

        tmp.fml <- update(fml, sprintf(". ~ . + %s", x.star))
        tmp.egg <- fit_formula(tmp.fml)
        tmp.level <- stopFun(tmp.egg)
        showiter(verbose, x.star, tmp.level)

        if (is.null(maxK) && (tmp.level >= level) )
            break

        fml <- tmp.fml
        egg <- tmp.egg
        level <- tmp.level

        if ( !is.null(maxK) && (length(all.vars(fml[[3]])) >= maxK) )
            break
    }

    egg <- fit_formula(fml)
    class(egg) <- c("frs", class(egg))
    return(egg)
}

#' @name pggm
#' @title Profile Boosting for Gaussian Graphical Model
#' 
#' @description Profile boosting for Gaussian graphical model.
#' 
#' @param S Covariance matrix.
#' @param nObs Number of observations.
#' @param maxK Maximum number of identified edges.
#' @param digits Integer indicating the number of decimal
#'  places or significant digits to be used.
#' @param verbose Print the procedure path?
#' 
#' @return Index set of identified features.
#' 
#' @examples
#' library(MASS)
#' library(Matrix)
#' 
#' set.seed(2025)
#' n <- 1000
#' p <- 10
#' 
#' Omega <- Diagonal(p)
#' diag(Omega[1:4, 2:5]) <- diag(Omega[2:5, 1:4]) <- 0.5
#' Sigma <- chol2inv(chol(Omega))
#' X <- mvrnorm(n, rep(0, p), Sigma, empirical=TRUE)
#' S <- cov(X)
#' system.time( egg <- pggm(S, n) )
#' 
NULL
#> NULL



#' @rdname pggm
#' @export
pggm <- function(S, nObs, maxK = floor(min(nObs-1, NROW(S)-1, 50)),
                digits = 8, verbose = FALSE) {
    stopifnot( isSymmetric(S) )
    stopifnot( nObs > 0 )
    stopifnot( maxK > 0 )

    S <- Matrix(S)

    p0 <- NROW(S)
    rEBIC <- max( 0.0, 1.0 - log(nObs) / (2.0 * log(p0)))

    grad <- function(Omega, nonzeroIdx) {
        egg <- S - inv.bdMat(Omega, nonzeroIdx)
        return(2*egg - Diagonal(x = diag(egg)))
    }
    ebic <- function(Omega, dof) {
        bic <- c(sum(S*Omega) - determinant(Omega)$modulus + dof * log(nObs) / nObs)
        return( bic + 2 * rEBIC * lchoose(p0 * (p0 - 1), dof) / nObs )
    }

    showiter <- function(verbose, k, idx, level)
        if (verbose)
            message(sprintf("Step %i: add (%i, %i) with level=%.3f)",
                            k, idx[1], idx[2], level))

    nonzeroIdx <- matrix(NA, 0, 2)
    Omega <- Diagonal(x = 1.0 / diag(S))
    ebicVal <- ebic(Omega, NROW(nonzeroIdx))
    ebicVec <- c(ebicVal, rep(NA_real_, maxK))
    k <- 1
    while (k <= maxK) {
        pps <- abs(grad(Omega, nonzeroIdx))
        idx.tmp <- sort(which(pps == max(pps), TRUE)[1, ])
        nonzeroIdx.tmp <- rbind(nonzeroIdx, idx.tmp)

        rmle.tmp <- rmle4ggmS4(S, nonzeroIdx.tmp)
        ebicVal.tmp <- ebic(rmle.tmp$Omega, NROW(nonzeroIdx.tmp))

        # message(sprintf("%f", ebicVal.tmp))

        if (ebicVal.tmp >= ebicVal) break

        nonzeroIdx <- nonzeroIdx.tmp
        Omega <- rmle.tmp$Omega
        ebicVal <- ebicVal.tmp
        
        k <- k + 1
        ebicVec[k+1] <- ebicVal
    }

    dimnames(nonzeroIdx) <- NULL
    return(list(
        Omega = round(Omega, digits),
        nonzeroIdx = nonzeroIdx,
        ebic = c(na.omit(ebicVec))
    ))
}






#' @rdname pggm
#' @description `rmle4ggmS4.dense`: Restricted MLE for GGM.
#' @noRd
rmle4ggmS4.dense <- function(S, nonzeroIdx, tol = 1e-4, maxIter = 100, verbose = FALSE) {
    stopifnot( isSymmetric(S) )
    stopifnot( all(nonzeroIdx > 0) )
    stopifnot( all(nonzeroIdx <= NROW(S)) )
    stopifnot( all(c(tol, maxIter) > 0) )

    active.mat <- Matrix(0.0, NROW(S), NCOL(S))
    active.mat[nonzeroIdx] <- 1.0
    active.mat <- active.mat + t(active.mat)
    diag(active.mat) <- 1.0

    loss <- function(Omega) c(sum(S * Omega) - determinant(Omega)$modulus)
    grad <- function(Omega, nonzeroIdx) {
        egg <- active.mat * (S - inv.bdMat(Omega, nonzeroIdx))
        return(2*egg - Diagonal(x = diag(egg)))
    }

    showiter <- function(verbose)
        if (verbose) message(sprintf("iter %i: obj=%.4f", k, obj))

    Omega0 <- Diagonal(x = 1.0/diag(S))
    Omega <- W <- Omega0
    obj <- loss(Omega0)

    alpha0 <- 1.0
    alpha <- (1.0 + sqrt(1.0 + 4*alpha0^2)) / 2.0
    iter <- rep(NA_real_, maxIter)
    isConvergent <- FALSE
    delta <- 1.0
    for (k in seq_len(maxIter)) {
        showiter(verbose)
        iter[k] <- obj

        ## search point
        W <- Omega + (alpha0 - 1.0)/alpha * (Omega - Omega0)

        ## approximate solution
        Omega0 <- Omega
        obj0 <- obj
        W.go <- grad(W, nonzeroIdx) / delta
        if (max(abs(W.go)) < min(1.0e-6, 0.1*tol)) {
            isConvergent <- TRUE
            break
        }

        Omega <- symmpart(nearPD(W - W.go)$mat) # `keepDiag`
        obj <- loss(Omega)
        stopifnot( isSymmetric(Omega) )

        # here maybe arise the case `obj - obj0 \approx tol > 0`
        if (obj > obj0) {
            Omega <- Omega0
            obj <- obj0
            delta <- 1.5 * delta
            next
        }

        if ( abs(obj-obj0) <= (abs(obj0) + 0.1)*tol ) {
            isConvergent <- TRUE
            break
        }

        ## update paras
        alpha0 <- alpha
        alpha <- (1.0 + sqrt(1.0 + 4*alpha0^2)) / 2.0
    }

    list(Omega = round(Omega, 8), iter = c(na.omit(iter)), isConvertent = isConvergent)
}



#' @rdname pggm
#' @description `rmle4ggmS4`: Restricted MLE for GGM
#' @noRd
rmle4ggmS4  <- function(S, nonzeroIdx, tol = 1e-4, maxIter = 100) {
    stopifnot( isSymmetric(S) )
    stopifnot( all(nonzeroIdx > 0) )
    stopifnot( all(nonzeroIdx <= NROW(S)) )
    stopifnot( all(c(tol, maxIter) > 0) )

    idx <- sort(unique(c(nonzeroIdx)))

    indicator.mat <- Matrix(FALSE, NROW(S), NCOL(S), doDiag = FALSE)
    indicator.mat[nonzeroIdx] <- TRUE
    nonzeroIdx.sub <- which(indicator.mat[idx, idx, drop = FALSE], TRUE)
    obj <- rmle4ggmS4.dense(S[idx, idx, drop = FALSE], nonzeroIdx.sub, tol, maxIter)

    Omega <- Diagonal(x = 1.0/diag(S))
    Omega[idx, idx] <- obj$Omega

    list(Omega = Omega, iter = obj$iter, isConvergent = obj$isConvergent)
}

#' @name pglm
#' @title Profile Boosting for Generalized Linear Models.
#' 
#' @description
#' [pglm] inherits the usage of the built-in function [glm].
#' 
#' @param formula Parameter passed to [glm].
#' @param data Parameter passed to [glm].
#' @param family Parameter passed to [glm].
#' @param weights Parameter passed to [glm].
#' @param subset Parameter passed to [glm].
#' @param na.action Parameter passed to [glm].
#' @param start Parameter passed to [glm].
#' @param etastart Parameter passed to [glm].
#' @param mustart Parameter passed to [glm].
#' @param offset Parameter passed to [glm].
#' @param control Parameter passed to [glm].
#' @param model Parameter passed to [glm].
#' @param method Parameter passed to [glm].
#' @param x Parameter passed to [glm].
#' @param y Parameter passed to [glm].
#' @param singular.ok Parameter passed to [glm].
#' @param contrasts Parameter passed to [glm].
#' @param ... Parameters passed to [glm].
#' @param intercept Parameter passed to [glm.fit].
#' 
#' @param stopFun Parameter passed to [pboost::pboost].
#' @param keep Parameter passed to [pboost::pboost].
#' @param maxK Parameter passed to [pboost::pboost].
#' @param verbose Parameter passed to [pboost::pboost].
#' 
#' @return A `glm` model object fitted on the selected features.
#' 
#' @references
#' Zengchao Xu, Shan Luo and Zehua Chen (2022). Partial profile score feature selection
#' in high-dimensional generalized linear interaction models. Statistics and Its Interface.
#' \doi{10.4310/21-SII706}
#' 
#' @examples
#' set.seed(2026)
#' n <- 200
#' p <- 100
#' x <- matrix(rnorm(n*p), n)
#' 
#' eta <- drop( x[, 1:3] %*% runif(3, 1.0, 1.5) )
#' y <- rbinom(n, 1, 1/(1+exp(-eta)))
#' DF <- data.frame(y, x)
#' 
#' ## ---------- pboost ----------
#' pglm(y ~ ., "binomial", DF, verbose=TRUE)
#' pglm(y ~ ., "binomial", DF, stopFun=BIC, verbose=TRUE)
#' 
#' scoreLogistic <- function(object) {
#'    eta.hat <- object[["linear.predictors"]]
#'    return(object[["y"]] - 1/(1+exp(-eta.hat)))
#' }
#' (result <- pboost(y, x, glm, scoreLogistic, family="binomial", verbose=TRUE))
#' all.vars(formula(result)[[3]])
#' 
#' ## ---------- frs ----------
#' fglm(y ~ ., "binomial", DF, verbose=TRUE)
#' fglm(y ~ ., "binomial", DF, stopFun=BIC, verbose=TRUE)
#' 
#' frs(y, x, glm, family="binomial", verbose=TRUE)
#' 
NULL
#> NULL



#' @rdname pglm
#' @order 1
#' @export
pglm <- function(formula, family = gaussian, data, weights, subset,
                na.action, start = NULL, etastart, mustart, offset,
                control = list(...), model = TRUE, method = "glm.fit",
                x = FALSE, y = TRUE, singular.ok = TRUE, contrasts = NULL, ...,
                stopFun = "EBIC",
                keep = NULL, maxK = NULL, verbose = FALSE) {

    fml <- Formula(formula)
    mf <- model.frame(fml, data=data)

    yvec <- model.part(fml, data=mf, lhs=1, drop=TRUE)
    xmat <- model.part(fml, data=mf, rhs=1) |> as.matrix()
    use.intercept <- attr(terms(mf), "intercept") == 1L
    # use.intercept <- attr(terms(fml, data=data), "intercept") == 1L


    scoreFun <- function(object) {
        # score <- D0/S0*(y-fitted(obj))
        # profilescore <- drop(crossprod(x, score))
        # profilescore.sd <- drop(sqrt(crossprod(x*x, D0^2/S0)))
        # return( profilescore / profilescore.sd )
        D0 <- object$family$mu.eta(object$linear.predictors)
        S0 <- object$family$variance(object$fitted.values)
        # weights <- object[["prior.weights"]]
        # return( weights*D0/S0 * (y-fitted(obj)) )
        # return( weights * D0 / S0 * residuals(object, type="response") )
        return( D0 / S0 * residuals(object, type = "response") )
    }

    mc <- match.call(expand.dots = TRUE)
    provided_args <- as.list(mc)[-1]
    provided_args <- provided_args[!(names(provided_args) %in% c("scoreFun"))]
    provided_args$formula <- NULL
    provided_args$data <- NULL

    args <- list(
        fitFun = glm,
        yvec = yvec,
        xmat = xmat,
        use.intercept = use.intercept,
        scoreFun = scoreFun
    )
    args <- c(args, provided_args)

    return(do.call(pboost, args))
}



#' @rdname pglm
#' @order 2
#' @export
pglm.fit <- function(x, y, weights = rep.int(1, NROW(y)), start = NULL, etastart = NULL,
                      mustart = NULL, offset = rep.int(0, NROW(y)), family = gaussian(),
                      control = list(), intercept = TRUE, singular.ok = TRUE,
                      stopFun = "EBIC",
                      keep = NULL, maxK = NULL, verbose = FALSE) {

    n <- NROW(x)
    p <- NCOL(x)

    scoreFun <- function(object) {
        class(object) <- "glm"
        D0 <- object$family$mu.eta(object$linear.predictors)
        S0 <- object$family$variance(object$fitted.values)
        return( D0 / S0 * residuals(object, type = "response") )
    }

    if (is.character(stopFun) && stopFun == "EBIC")
        stopFun <- function(object) {
            class(object) <- "glm"

            ebic.r <- max( 0.0, 1.0 - log(n) / (2.0 * log(p)) )
            stopifnot( ebic.r >= 0 )

            dof <- length(coef(object))
            ebic.penalty <- 2.0 * ebic.r * lchoose(p - length(keep), dof - length(keep))
            stopifnot( is.finite(ebic.penalty) )

            return(BIC(object) + ebic.penalty)
        }


    mc <- match.call(expand.dots = TRUE)
    provided_args <- as.list(mc)[-1]
    provided_args <- provided_args[!(names(provided_args) %in% c("scoreFun", "stopFun"))]
    provided_args$x <- NULL
    provided_args$y <- NULL

    args <- list(
        fitFun = glm.fit,
        yvec = y,
        xmat = x,
        scoreFun = scoreFun,
        stopFun = stopFun,
        use.formula = FALSE
    )
    args <- c(args, provided_args)

    return(do.call(pboost, args))
}

#' @rdname penalization
#' 
#' @description
#' - `pen_rq(x, y, tau, penalty)`: penalized quantile regression models.
#' 
#' @noRd
pen_rq <- function(x, y, tau, penalty) {
    stopifnot( penalty %in% c("MCP", "SCAD", "lasso") )
    penalty <- toupper(penalty)

    # `rq.pen.cv`, `rq.pen`
    egg <- rq.pen(
        x = x,
        y = y,
        tau = tau,
        penalty = penalty,
        scalex = FALSE
    )

    class(egg) <- c("penaltyrq", class(egg))
    return(egg)
}



#' @title Extract Coefficients from Non-Convex Penalized Models
#'
#' @param object Object.
#' @param ... Additional arguments (not used).
#' 
#' @return A named vector of coefficients.
#' 
#' @noRd
coef.penaltyrq <- function(object, ...) {
    class(object) <- setdiff(class(object), "penaltyrq")
    b0 <- qic.select(object, method="PBIC") |> coef() |> drop()
    b0 <- b0[grep("intercept", names(b0), value=TRUE, fixed=TRUE, invert=TRUE)]
    return(b0[abs(b0) >= 1e-4])
}

#' @rdname penalization
#' 
#' @description
#' - `pen_ncvreg(x, y, family, penalty)`: Non-Convex penalized models.
#' 
#' @noRd
pen_ncvreg <- function(x, y, family, penalty) {
    stopifnot( family %in% c("gaussian", "binomial", "cox") )
    stopifnot( penalty %in% c("MCP", "SCAD", "lasso") )

    if (family == "cox") {
        egg <- cv.ncvsurv(
            X = x,
            y = y,
            penalty = penalty,
            nfolds = 10
        )
    } else {
        egg <- cv.ncvreg(
            X = x,
            y = y,
            family = family,
            penalty = penalty,
            nfolds = 10
        )
    }

    class(egg) <- c("penaltyncvreg", class(egg))
    return(egg)
}



#' @title Extract Coefficients from Non-Convex Penalized Models
#'
#' @param object Object.
#' @param ... Additional arguments (not used).
#' 
#' @return Named vector of non-zero coefficients of under cross-validation error.
#' 
#' @noRd
coef.penaltyncvreg <- function(object, ...) {
    idx <- object[["min"]]
    b0 <- object[["fit"]][["beta"]][, idx, drop=TRUE]
    b0 <- b0[grep("(Intercept)", names(b0), value=TRUE, fixed=TRUE, invert=TRUE)]
    return(b0[abs(b0) >= 1e-4])
}

#' @name pbetareg
#' @title Profile Boosting for Beta Regression
#' 
#' @description
#' [pbetareg] inherits the usage of [betareg::betareg].
#' 
#' @param formula Parameter passed to [betareg::betareg].
#' @param data Parameter passed to [betareg::betareg].
#' @param subset Parameter passed to [betareg::betareg].
#' @param na.action Parameter passed to [betareg::betareg].
#' @param weights Parameter passed to [betareg::betareg].
#' @param offset Parameter passed to [betareg::betareg].
#' @param link Parameter passed to [betareg::betareg].
#' @param link.phi Parameter passed to [betareg::betareg].
#' @param type Parameter passed to [betareg::betareg].
#' @param dist Parameter passed to [betareg::betareg].
#' @param nu Parameter passed to [betareg::betareg].
#' @param control Parameter passed to [betareg::betareg].
#' @param model Parameter passed to [betareg::betareg].
#' @param y Parameter passed to [betareg::betareg].
#' @param x Parameter passed to [betareg::betareg].
#' @param ... Parameters passed to [betareg::betareg].
#' 
#' @param stopFun Parameter passed to [pboost::pboost].
#' @param keep Parameter passed to [pboost::pboost].
#' @param maxK Parameter passed to [pboost::pboost].
#' @param verbose Parameter passed to [pboost::pboost].
#' 
#' @return A `betareg` model object fitted on the selected features.
#' 
#' @examples
#' \dontrun{
#' set.seed(2026)
#' n <- 300
#' p <- 20
#' x <- matrix(runif(n*p), n)
#' mu <- runif(n)
#' phi <- 1.0
#' 
#' shape1 <- mu * phi
#' shape2 <- (1-mu) * phi
#' y <- rbeta(n, shape1, shape2)
#' DF <- data.frame(y, x)
#' 
#' pbetareg(y ~ ., DF, verbose=TRUE)
#' fbetareg(y ~ ., DF, verbose=TRUE)
#' }
#' 
NULL
#> NULL


#' @rdname pbetareg
#' @order 1
#' @export
pbetareg <- function(formula, data, subset, na.action, weights, offset,
                    link = c("logit", "probit", "cloglog", "cauchit", "log","loglog"),
                    link.phi = NULL, type = c("ML", "BC", "BR"),
                    dist = NULL, nu = NULL, control = betareg.control(...),
                    model = TRUE, y = TRUE, x = FALSE, ...,
                    stopFun = "EBIC",
                    keep = NULL, maxK = NULL, verbose = FALSE) {

    fml <- Formula(formula)
    mf <- model.frame(fml, data=data)

    yvec <- model.part(fml, data=mf, lhs=1, drop=TRUE)
    xmat <- model.part(fml, data=mf, rhs=1) |> as.matrix()
    use.intercept <- attr(terms(mf), "intercept") == 1L


    scoreFun <- function(object) {
        phi <- predict(object, type='precision')
        mu <- predict(object, type='response')
        eta <- predict(object, type='link')

        ## object$link is a list of two elements: one for "mean" or "mu"
        mu.eta <- object$link[[grep("^m", names(object$link), value=TRUE)]]$mu.eta
        y <- pmin(pmax(object[["y"]], .Machine$double.eps), 1 - .Machine$double.eps)

        # weights <- object[["weights"]]
        return( mu.eta(eta) * phi * ( digamma((1-mu)*phi) - digamma(mu*phi) + qlogis(y) ) )
    }

    mc <- match.call(expand.dots = TRUE)
    provided_args <- as.list(mc)[-1]
    provided_args <- provided_args[!(names(provided_args) %in% c("scoreFun"))]
    provided_args$formula <- NULL
    provided_args$data <- NULL

    args <- list(
        fitFun = betareg,
        yvec = yvec,
        xmat = xmat,
        use.intercept = use.intercept,
        scoreFun = scoreFun
    )
    args <- c(args, provided_args)

    return(do.call(pboost, args))
}

#' @name pcoxph
#' @title Profile Boosting for Cox proportional hazards Model
#' 
#' @description Profile boosting for Cox model.
#' 
#' @param formula Parameter passed to [survival::coxph].
#' @param data Parameter passed to [survival::coxph].
#' @param weights Parameter passed to [survival::coxph].
#' @param subset Parameter passed to [survival::coxph].
#' @param na.action Parameter passed to [survival::coxph].
#' @param init Parameter passed to [survival::coxph].
#' @param control Parameter passed to [survival::coxph].
#' @param ties Parameter passed to [survival::coxph].
#' @param singular.ok Parameter passed to [survival::coxph].
#' @param robust Parameter passed to [survival::coxph].
#' @param model Parameter passed to [survival::coxph].
#' @param x Parameter passed to [survival::coxph].
#' @param y Parameter passed to [survival::coxph].
#' @param tt Parameter passed to [survival::coxph].
#' @param method Parameter passed to [survival::coxph].
#' @param id Parameter passed to [survival::coxph].
#' @param cluster Parameter passed to [survival::coxph].
#' @param istate Parameter passed to [survival::coxph].
#' @param statedata Parameter passed to [survival::coxph].
#' @param nocenter Parameter passed to [survival::coxph].
#' @param ... Parameters passed to [survival::coxph].
#' 
#' @param stopFun Parameter passed to [pboost::pboost].
#' @param keep Parameter passed to [pboost::pboost].
#' @param maxK Parameter passed to [pboost::pboost].
#' @param verbose Parameter passed to [pboost::pboost].
#' 
#' @return A `coxph` model object fitted on the selected features.
#' 
#' @examples
#' library(survival)
#' set.seed(2026)
#' n <- 300
#' p <- 200
#' 
#' DF <- data.frame(
#'     time = rpois(n, 5),
#'     status = rbinom(n, 1, 0.3),
#'     matrix(rnorm(n*p), n)
#' )
#' 
#' pcoxph(Surv(time, status) ~ ., DF, verbose=TRUE)
#' fcoxph(Surv(time, status) ~ ., DF, verbose=TRUE)
#' 
NULL
#> NULL



#' @rdname pcoxph
#' @order 1
#' @export
pcoxph <- function(formula, data, weights, subset, na.action, init, control,
                    ties = c("efron", "breslow", "exact"), singular.ok = TRUE,
                    robust, model = FALSE, x = FALSE, y = TRUE, tt, method = ties,
                    id, cluster, istate, statedata, nocenter = c(-1, 0, 1), ...,
                    stopFun = "EBIC",
                    keep = NULL, maxK = NULL, verbose = FALSE) {
    fml <- Formula(formula)
    mf <- model.frame(fml, data=data)

    yvec <- model.part(fml, data=mf, lhs=1, drop=TRUE)
    xmat <- model.part(fml, data=mf, rhs=1) |> as.matrix()
    use.intercept <- FALSE


    mc <- match.call(expand.dots = TRUE)
    provided_args <- as.list(mc)[-1]
    provided_args <- provided_args[!(names(provided_args) %in% c("scoreFun"))]
    provided_args$formula <- NULL
    provided_args$data <- NULL

    args <- list(
        fitFun = coxph,
        yvec = yvec,
        xmat = xmat,
        use.intercept = use.intercept,
        scoreFun = scoreCoxph
    )
    args <- c(args, provided_args)

    return(do.call(pboost, args))
}



#' @rdname pcoxph
#' @noRd
scoreCoxph <- function(object) {
    eta <- as.numeric(object$linear.predictors)
    method <- object$method
    stopifnot( method %in% c("efron", "breslow") )

    y <- object$y
    if (is.null(y)) {
        y <- model.response(model.frame(object))
    }
    if (inherits(y, 'Surv')) {
        time <- y[,1]
        status <- y[,ncol(y)]
    } else {
        stop('Cannot recognize y')
    }

    if (!(length(time) == length(status) && length(status) == length(eta)))
        stop('length mismatch')
    n <- length(time)
    o <- order(time, decreasing = TRUE)
    time_o <- time[o]
    status_o <- as.integer(status[o])
    eta_o <- eta[o]
    w <- exp(eta_o)
    rle_time <- rle(time_o)
    lengths <- rle_time$lengths
    ends <- cumsum(lengths)
    starts <- ends - lengths + 1

    grad_o <- numeric(n)

    if (method == 'breslow') {
        # Breslow: for each event block add d / S to all at-risk entries
        cw <- cumsum(w) # cumulative weight over decreasing times
        A_o <- numeric(n)
        for (k in seq_along(ends)) {
            s <- starts[k]; e <- ends[k]
            d <- sum(status_o[s:e] == 1)
            if (d == 0) next

            S <- cw[e]
            A_o[1:e] <- A_o[1:e] + (d / S)
        }
        grad_o <- status_o - w * A_o
    } else if (method == 'efron') {
        # Efron: more careful handling of ties with fractional removal
        for (k in seq_along(ends)) {
            s <- starts[k]; e <- ends[k]
            d <- sum(status_o[s:e] == 1)
            if (d == 0) next

            deaths <- which(status_o[s:e] == 1) + s - 1
            riskset <- 1:e
            S0 <- sum(w[riskset])
            sum_deaths <- sum(w[deaths])

            # for l = 0..(d-1), partially remove l/d of the deaths
            for (l in 0:(d-1)) {
                S0l <- S0 - (l / d) * sum_deaths

                # deaths contribute +1/d each in status part; then subtract w/S0l
                for (j in deaths)
                    grad_o[j] <- grad_o[j] + 1/d - w[j] / S0l

                # non-deaths in riskset only get - w / S0l contribution
                non_deaths <- setdiff(riskset, deaths)
                if (length(non_deaths))
                    grad_o[non_deaths] <- grad_o[non_deaths] - w[non_deaths] / S0l
            }
        }
    }

    grad <- numeric(n)
    grad[o] <- grad_o
    grad
}

#' @name penalization
#' 
#' @title Penalized Methods for Feature Selection
#' 
#' @description Penalized methods for feature selection.
#' 
#' @param x Feature matrix
#' @param y Response vector.
#' @param family "gaussian", "binomial", or "cox".
#' @param penalty "lasso", "SCAD" or "MCP".
#' @param tau Quantiles to be modeled.
#' @param rho Spatial autoregressive parameter. If missing or NULL, it will be estimated.
#' @param w Weight matrix (row-sum scaled being one).
#' 
#' @return Set of selected features.
#' 
#' @examples
#' library(survival)
#' set.seed(2026)
#' n <- 10
#' p <- 20
#' 
#' x <- replicate(p, rnorm(n))
#' b0 <- runif(3, 1.5, 2.0)
#' eta <- drop(x[, 1:3] %*% b0)
#' 
#' ## ---------- linear ----------
#' y <- rnorm(n, eta)
#' 
#' pen_glmnet(x=x, y=y, family="gaussian") |> coef()
#' pen_ncvreg(x=x, y=y, family="gaussian", penalty="MCP") |> coef()
#' pen_ncvreg(x=x, y=y, family="gaussian", penalty="SCAD") |> coef()
#' 
#' 
#' ## ---------- logistic ----------
#' y <- rbinom(n, 1, 1.0 / (1.0 + exp(-eta)))
#' 
#' pen_glmnet(x=x, y=y, family="binomial") |> coef()
#' pen_ncvreg(x=x, y=y, family="binomial", penalty="MCP") |> coef()
#' pen_ncvreg(x=x, y=y, family="binomial", penalty="SCAD") |> coef()
#' 
#' 
#' ## ---------- cox ----------
#' h0 <- 0.01
#' censoringRate <- 0.3
#' survivalTime <- -log(runif(n)) / (h0 * exp(eta))
#' censoringTime <- rexp(n, rate = -log(1 - censoringRate)/median(survivalTime))
#' y <- cbind(
#'     time = pmin(survivalTime, censoringTime),
#'     status = as.numeric(survivalTime <= censoringTime)
#' )
#' 
#' pen_glmnet(x=x, y=y, family="cox") |> coef()
#' pen_ncvreg(x=x, y=y, family="cox", penalty="MCP") |> coef()
#' pen_ncvreg(x=x, y=y, family="cox", penalty="SCAD") |> coef()
#' 
#' 
#' ## ---------- quantile ----------
#' y <- eta + rt(n, 2)
#' 
#' pen_rq(x=x, y=y, tau=0.5, penalty="lasso") |> coef()
#' pen_rq(x=x, y=y, tau=0.5, penalty="MCP") |> coef()
#' pen_rq(x=x, y=y, tau=0.5, penalty="SCAD") |> coef()
#' 
#' 
#' ## ---------- sar ----------
#' w0 <- set_rook_matrix(5, n/5)
#' rho0 <- 0.5
#' y <- solve(diag(n) - rho0 * w0, rnorm(n, eta))
#' 
#' pen_sar(x=x, y=y, rho=rho0, w=w0, penalty="lasso") |> coef()
#' pen_sar(x=x, y=y, rho=rho0, w=w0, penalty="MCP") |> coef()
#' pen_sar(x=x, y=y, rho=rho0, w=w0, penalty="SCAD") |> coef()
#' 
#' @noRd
#' 
NULL
#> NULL

#' @title Generalized Inverse
#' @description Generalized inverse of a matrix with form `DiagBlock(dense, diag)`.
#' @param x Matrix
#' @param nonzeroIdx Index of nonzero entries.
#' @noRd
inv.bdMat <- function(x, nonzeroIdx) {
    idx <- sort(unique(c(nonzeroIdx)))

    x.inv <- Diagonal(x=1.0/diag(x))
    if (NROW(nonzeroIdx) > 0)
        x.inv[idx, idx] <- ginv(nearPD(x[idx, idx, drop=FALSE], base.matrix=TRUE)$mat)

    return(x.inv)
}

#' @name frs
#' @title Forward Regression Selection Framework
#' 
#' @description `frs()` is a generic workhorse function of forward regression selection for parametric regression.
#' 
#' @param yvec See [pboost].
#' @param xmat See [pboost].
#' @param fitFun See [pboost].
#' @param ... See [pboost].
#' @param use.formula See [pboost].
#' @param use.intercept See [pboost].
#' @param selectFun A function to evaluate the importance of an unselected feature when it is added to current model.
#' The default is `logLik`, meaning that the feature with the largest post-added log-likelihood is identified as the next one to be added to the model.
#' Note that `selectFun` is only used for selecting features, and it does not affect the stopping rule of forward regression selection, which is determined by `stopFun`.
#' @param stopFun See [pboost].
#' @param maxK See [pboost].
#' @param keep See [pboost].
#' @param verbose See [pboost].
#' 
#' @return Model object fitted on the selected features.
#' 
#' @seealso [pboost::fbetareg], [pboost::fcoxph], [pboost::fglm], [pboost::flm], [pboost::frq], [pboost::fsar].
#' 
NULL
#> NULL


#' @rdname frs
#' @order 1
#' @export
frs <- function(yvec, xmat, fitFun, ...,
                use.formula = TRUE,
                use.intercept = TRUE,
                selectFun = logLik,
                stopFun = "EBIC",
                keep = NULL, maxK = NULL, verbose = FALSE) {
    stopifnot( NROW(yvec) == NROW(xmat) )
    stopifnot( is.matrix(xmat), is.numeric(xmat) )

    n <- NROW(xmat)
    p <- NCOL(xmat)

    xnames <- colnames(xmat)
    if (is.null(xnames)) {
        xnames <- paste0("x", seq_len(p))
        colnames(xmat) <- xnames
    }
    stopifnot( length(xnames) == p )

    if ( is.character(stopFun) && use.formula && stopFun == "EBIC")
        stopFun <- function(object){
            ebic.r <- max( 0.0, 1.0 - log(n) / (2.0 * log(p)) )
            stopifnot( ebic.r >= 0 )

            dof <- length(all.vars(formula(object)[[3]]))
            ebic.penalty <- 2.0 * ebic.r * lchoose(p - length(keep), dof - length(keep))
            stopifnot( is.finite(ebic.penalty) )

            # return(-2*logLik(object) + dof * log(n) + ebic.penalty)
            return(BIC(object) + ebic.penalty)
        }

    stopifnot( is.character(keep) || is.numeric(keep) || is.null(keep) )
    if (!is.null(keep)) {
        if (is.character(keep)) {
            stopifnot( all(keep %in% xnames) )
        } else {
            stopifnot( all(keep %in% seq_len(p)) )
            keep <- xnames[keep]
        }
    }
    stopifnot( is.character(keep) || is.null(keep) )

    if (!is.null(maxK))
        maxK <- min( maxK, p, n-1 )

    showiter <- function(verbose, x.star, level) {
        if (verbose)
            if (missing(x.star))
                message(sprintf("Initial model: level=%.3f", level))
            else 
                message(sprintf("Adding %s: level=%.3f", x.star, level))
    }

    fit_formula <- function(fml) {
        if (use.formula) {
            return(fitFun(
                formula = fml,
                data = data.frame(yvec, xmat[, all.vars(fml[[3]]), drop=FALSE]),
                ...
            ))
        } else {
            return(fitFun(x = xmat[, all.vars(fml[[3]]), drop=FALSE], y = yvec, ...))
        }
    }

    lhs <- "yvec"
    rhs <- paste(c(as.integer(use.intercept), keep), collapse=" + ")
    fml <- as.formula(paste(c(lhs, rhs), collapse=" ~ "))
    egg <- fit_formula(fml)
    level <- stopFun(egg)
    showiter(verbose, level=level)

    if ( !is.null(maxK) && !is.null(keep) && (length(keep) >= maxK) ) {
        warning("'length(keep)' is not less than 'maxK', thus fitting on 'keep'.")
        return(egg)
    }

    while (TRUE) {
        candidates <- setdiff(xnames, all.vars(fml[[3]]))
        if (length(candidates) == 0)
            break

        importantanceCand <- rep(-Inf, length(candidates))
        levelCand <- rep(NA_real_, length(candidates))
        for (iC in seq_along(candidates)) {
            tmp.fml <- update(fml, sprintf(". ~ . + %s", candidates[iC]))
            tmp.egg <- fit_formula(tmp.fml)
            importantanceCand[iC] <- selectFun(tmp.egg)
            levelCand[iC] <- stopFun(tmp.egg)
        }

        j.star <- which.max(importantanceCand)
        x.star <- candidates[j.star]
        stopifnot( length(x.star) == 1 )
        stopifnot( !(x.star %in% all.vars(fml[[3]])) )

        tmp.fml <- update(fml, sprintf(". ~ . + %s", x.star))
        tmp.level <- levelCand[j.star]
        showiter(verbose, x.star, tmp.level)

        if (is.null(maxK) && (tmp.level >= level) )
            break

        fml <- tmp.fml
        level <- tmp.level

        if ( !is.null(maxK) && (length(all.vars(fml[[3]])) >= maxK) )
            break
    }

    egg <- fit_formula(fml)
    class(egg) <- c("frs", class(egg))
    return(egg)
}

#' @name sar.model
#' @title Functions for Spatial Auto-regressive Model
#' 
#' @description
#' - `get_rho()`: MLE of \eqn{\rho} in SAR model. See [pboost::psar].
#' - `sar.fit()`: Fit SAR model with given \eqn{\rho}.
#' - `set_rook_matrix()`: Construct Rook weight adjacency matrix.
#' - `logLik()`, `BIC()`, `residuals()`, `coef()`.
#' 
#' @param x Numeric feature matrix.
#' @param y Response vector.
#' @param w Weight matrix (row-sum scaled).
#' @param rNum Number of row units in Rook structure.
#' @param cNum Number of column units in Rook structure.
#' @param object Object.
#' @param ... Arguments.
#' 
#' @return Value or list.
#' 
NULL
#> NULL


#' @name sar.model
#' @export
get_rho <- function(x, y, w) {
    stopifnot( is.matrix(x) )
    stopifnot( NROW(x) == length(y) )
    stopifnot( all( abs(rowSums(w) - 1.0) <= 1e-12 ) )
    stopifnot( all(w >= 0.0) )

    if (NCOL(x) > NROW(x))
        warning("Singularity Possible!")

    minusloglik <- function(rho) {
        A.rho <- diag(NROW(x)) - rho * w
        sig2.hat <- mean( residuals(lm.fit(x, A.rho %*% y))^2 ) # singular.ok=FALSE
        0.5*NROW(x) * log(sig2.hat) - log(det(A.rho))
    }

    # return(optimize(minusloglik, c(-1, 1))$minimum)
    rho.min <- optimize(minusloglik, c(-1, 1))
    egg <- rho.min$minimum
    attr(egg, "minusloglik") <- rho.min$objective
    return(egg)
}

#' @name sar.model
#' @export
sar.fit <- function(x, y, w) {
    stopifnot( NROW(x) == length(y) )
    stopifnot( is.matrix(x) )

    rho.hat  <- get_rho(x, y, w)
    y.tilde  <- y - rho.hat * (w %*% y)
    beta.hat <- coef(lm.fit(x, y.tilde, singular.ok = FALSE))
    err      <- drop(y.tilde - x %*% beta.hat)
    sig2.hat <- mean( err^2 )

    egg <- list(
        beta = beta.hat,
        sig2 = sig2.hat,
        rho = rho.hat,
        residuals = err
    )
    attr(egg, "dof") <- NCOL(x)
    attr(egg, "nobs") <- NROW(x)
    attr(egg, "minusloglik") <- attr(rho.hat, "minusloglik")
    attr(egg, "bic") <- 2 * attr(rho.hat, "minusloglik") + log(NROW(x)) * NCOL(x)

    class(egg) <- "sarpboost"
    return(egg)
}


#' @name sar.model
#' @export
residuals.sarpboost <- function(object, ...)
    return(object[["residuals"]])


#' @name sar.model
#' @export
logLik.sarpboost <- function(object, ...)
    return(-1.0*attr(object, "minusloglik"))


#' @name sar.model
#' @export
BIC.sarpboost <- function(object, ...)
    return(attr(object, "bic"))


#' @name sar.model
#' @export
coef.sarpboost <- function(object, ...)
    return(object[["beta"]])



#' @name sar.model
#' @export
set_rook_matrix <- function(rNum, cNum) {
    idxR <- as.vector( row(matrix(NA, rNum, cNum)) )
    idxC <- as.vector( col(matrix(NA, rNum, cNum)) )

    flag.row <- ( outer(idxR, idxR, \(i, j) i == j) & outer(idxC, idxC, \(i, j) abs(i - j) == 1) )
    flag.col <- ( outer(idxC, idxC, \(i, j) i == j) & outer(idxR, idxR, \(i, j) abs(i - j) == 1) )
    w <- (flag.row | flag.col) * 1.0

    return(w / rowSums(w))
}

#' @name fcoxph
#' @title Forward Regression Selection for Cox proportional hazards Model
#' 
#' @description Forward regression selection for Cox model.
#' 
#' @param formula Parameter passed to [survival::coxph].
#' @param data Parameter passed to [survival::coxph].
#' @param weights Parameter passed to [survival::coxph].
#' @param subset Parameter passed to [survival::coxph].
#' @param na.action Parameter passed to [survival::coxph].
#' @param init Parameter passed to [survival::coxph].
#' @param control Parameter passed to [survival::coxph].
#' @param ties Parameter passed to [survival::coxph].
#' @param singular.ok Parameter passed to [survival::coxph].
#' @param robust Parameter passed to [survival::coxph].
#' @param model Parameter passed to [survival::coxph].
#' @param x Parameter passed to [survival::coxph].
#' @param y Parameter passed to [survival::coxph].
#' @param tt Parameter passed to [survival::coxph].
#' @param method Parameter passed to [survival::coxph].
#' @param id Parameter passed to [survival::coxph].
#' @param cluster Parameter passed to [survival::coxph].
#' @param istate Parameter passed to [survival::coxph].
#' @param statedata Parameter passed to [survival::coxph].
#' @param nocenter Parameter passed to [survival::coxph].
#' @param ... Parameters passed to [survival::coxph].
#' 
#' @param selectFun Parameter passed to [pboost::frs].
#' @param stopFun Parameter passed to [pboost::frs].
#' @param keep Parameter passed to [pboost::frs].
#' @param maxK Parameter passed to [pboost::frs].
#' @param verbose Parameter passed to [pboost::frs].
#' 
#' @return A `coxph` model object fitted on the selected features.
#' 
NULL
#> NULL



#' @rdname fcoxph
#' @order 1
#' @export
fcoxph <- function(formula, data, weights, subset, na.action, init, control,
                    ties = c("efron", "breslow", "exact"), singular.ok = TRUE,
                    robust, model = FALSE, x = FALSE, y = TRUE, tt, method = ties,
                    id, cluster, istate, statedata, nocenter = c(-1, 0, 1), ...,
                    selectFun = logLik, stopFun = "EBIC",
                    keep = NULL, maxK = NULL, verbose = FALSE) {
    fml <- Formula(formula)
    mf <- model.frame(fml, data=data)

    yvec <- model.part(fml, data=mf, lhs=1, drop=TRUE)
    xmat <- model.part(fml, data=mf, rhs=1) |> as.matrix()
    use.intercept <- FALSE


    mc <- match.call(expand.dots = TRUE)
    provided_args <- as.list(mc)[-1]
    provided_args$formula <- NULL
    provided_args$data <- NULL

    args <- list(
        fitFun = coxph,
        yvec = yvec,
        xmat = xmat,
        use.intercept = use.intercept
    )
    args <- c(args, provided_args)

    return(do.call(frs, args))
}

#' @name EBIC
#' @title Extended Bayesian Information Criterion
#' @description The Extended BIC possesses the selection consistency in
#' high-dimensional model.
#'
#' It can be called by the fitted model that has standard [logLik] method
#' to access the attributes `nobs` and `df`, such as [lm], [glm].
#' 
#' @param object Fitted model object.
#' @param p Total number of candidate features, which is available in [pboost].
#' @param p.keep Number of features that are pre-specified to be kept in model.
#' @param ... Additional parameters, which is available in [pboost].
#' 
#' @return A function to obtain the EBIC value of a fitted object.
#' 
#' @details
#' The extended BIC (EBIC) is defined as
#' ```
#' EBIC(obj) = BIC(obj) + 2 * r * log(choose(p - |p.keep|, df - |p.keep|)).
#' ```
#' 
#' @references
#' * Jiahua Chen and Zehua Chen (2008). Extended Bayesian information criteria for
#' model selection with large model spaces. Biometrika, 95(3):759–771.
#' \doi{10.1093/biomet/asn034}
#' 
#' * Jiahua Chen and Zehua Chen (2012). Extended BIC for small-n-large-p sparse GLM.
#' Statistical Sinica, 22(2):555–574. \doi{10.5705/ss.2010.216}
#' 
NULL
#> NULL


#' @rdname EBIC
#' @order 1
#' @export
EBIC <- function(object, p, p.keep, ...) {
    UseMethod("EBIC")
}



#' @rdname EBIC
#' @export
#' @noRd
EBIC.default <- function(object, p, p.keep = NULL, ...) {
    stop("No EBIC method for object of class ", paste(class(object), collapse=", "))
}

#' @keywords internal
"_PACKAGE"

## usethis namespace: start
#' 
#' @import Formula
#' @import Matrix
#' @importFrom betareg betareg
#' @importFrom betareg betareg.control
#' @importFrom glmnet cv.glmnet
#' @importFrom glmnet glmnet
#' @importFrom MASS ginv
#' @importFrom ncvreg cv.ncvreg
#' @importFrom ncvreg cv.ncvsurv
#' @importFrom quantreg rq
#' @importFrom rqPen qic.select
#' @importFrom rqPen rq.pen
#' @importFrom stats AIC
#' @importFrom stats as.formula
#' @importFrom stats BIC
#' @importFrom stats coef
#' @importFrom stats fitted
#' @importFrom stats formula
#' @importFrom stats gaussian
#' @importFrom stats glm
#' @importFrom stats glm.fit
#' @importFrom stats lm
#' @importFrom stats lm.fit
#' @importFrom stats logLik
#' @importFrom stats model.frame
#' @importFrom stats model.response
#' @importFrom stats na.omit
#' @importFrom stats nobs
#' @importFrom stats optimize
#' @importFrom stats predict
#' @importFrom stats qlogis
#' @importFrom stats residuals
#' @importFrom stats terms
#' @importFrom survival coxph
#> [1] "_PACKAGE"
## usethis namespace: end
NULL

#' @name prq
#' @title Profile Boosting for Quantile Regression Models
#' 
#' @description
#' [prq] inherits the usage of the function [quantreg::rq].
#' 
#' @param formula Parameter passed to [quantreg::rq].
#' @param tau Parameter passed to [quantreg::rq].
#' @param data Parameter passed to [quantreg::rq].
#' @param subset Parameter passed to [quantreg::rq].
#' @param weights Parameter passed to [quantreg::rq].
#' @param na.action Parameter passed to [quantreg::rq].
#' @param method Parameter passed to [quantreg::rq].
#' @param model Parameter passed to [quantreg::rq].
#' @param contrasts Parameter passed to [quantreg::rq].
#' @param ... Parameters passed to [quantreg::rq].
#' 
#' @param stopFun Parameter passed to [pboost::pboost].
#' @param keep Parameter passed to [pboost::pboost].
#' @param maxK Parameter passed to [pboost::pboost].
#' @param verbose Parameter passed to [pboost::pboost].
#' 
#' @return A `rq` model object fitted on the selected features.
#' 
#' @examples
#' library(quantreg)
#' set.seed(2026)
#' n <- 300
#' p <- 20
#' x <- matrix(rnorm(n*p), n)
#' 
#' eta <- drop( x[, 1:3] %*% runif(3, 1.0, 1.5) )
#' y <- eta + (1.0 + x[, 3]) * rnorm(n)
#' DF <- data.frame(y, x)
#' 
#' tau <- 0.5
#' prq(y ~ ., tau, DF, verbose=TRUE)
#' 
#' BIC <- function(obj) AIC(obj, k=-1)
#' prq(y ~ ., tau, DF, stopFun=BIC, verbose=TRUE)
#' frq(y ~ ., tau, DF, stopFun=BIC, verbose=TRUE)
#' 
#' scoreFun <- function(object)
#'    return(ifelse(object[["y"]] < fitted(object), tau - 1, tau))
#'
#' pboost(y, x, rq, scoreFun, BIC, tau=tau, verbose=TRUE)
#' 
NULL
#> NULL



#' @rdname prq
#' @order 1
#' @export
prq <- function(formula, tau = 0.5, data, subset, weights, na.action,
                method = "br", model = TRUE, contrasts = NULL, ...,
                stopFun = "EBIC",
                keep = NULL, maxK = NULL, verbose = FALSE) {

    fml <- Formula(formula)
    mf <- model.frame(fml, data=data)

    yvec <- model.part(fml, data=mf, lhs=1, drop=TRUE)
    xmat <- model.part(fml, data=mf, rhs=1) |> as.matrix()
    use.intercept <- attr(terms(mf), "intercept") == 1L


    scoreFun <- function(object)
        return(ifelse(object[["y"]] < fitted(object), tau - 1, tau))

    n <- NROW(xmat)
    p <- NCOL(xmat)
    if (is.character(stopFun) && stopFun == "EBIC")
        stopFun <- function(object) {
            ebic.r <- max( 0.0, 1.0 - log(n) / (2.0 * log(p)) )
            stopifnot( ebic.r >= 0 )

            dof <- attr(logLik(object), "df")
            ebic.penalty <- 2.0 * ebic.r * lchoose(p - length(keep), dof - length(keep))
            stopifnot( is.finite(ebic.penalty) )

            # `BIC` for class `rq` is equivalent to `AIC` with negative `k`
            # return( -2*as.numeric(obj.loglik) + dof * log(n) + ebic.penalty )
            return( AIC(object, k=-1) + ebic.penalty )
        }

    mc <- match.call(expand.dots = TRUE)
    provided_args <- as.list(mc)[-1]
    provided_args <- provided_args[!(names(provided_args) %in% c("scoreFun", "stopFun"))]
    provided_args$formula <- NULL
    provided_args$data <- NULL

    args <- list(
        fitFun = rq,
        yvec = yvec,
        xmat = xmat,
        use.intercept = use.intercept,
        scoreFun = scoreFun,
        stopFun = stopFun
    )
    args <- c(args, provided_args)

    return(do.call(pboost, args))
}

#' @name plm
#' @title Profile Boosting for Linear Models.
#' 
#' @description
#' [plm] inherits the usage of the built-in function [lm].
#' 
#' @param formula Parameter passed to [lm].
#' @param data Parameter passed to [lm].
#' @param subset Parameter passed to [lm].
#' @param weights Parameter passed to [lm].
#' @param na.action Parameter passed to [lm].
#' @param method Parameter passed to [lm].
#' @param model Parameter passed to [lm].
#' @param x Parameter passed to [lm].
#' @param y Parameter passed to [lm].
#' @param qr Parameter passed to [lm].
#' @param singular.ok Parameter passed to [lm].
#' @param contrasts Parameter passed to [lm].
#' @param offset Parameter passed to [lm] or [lm.fit].
#' @param ... Parameters passed to [lm] or [lm.fit].
#' @param tol Parameter passed to [lm.fit].
#' 
#' @param stopFun Parameter passed to [pboost::pboost].
#' @param keep Parameter passed to [pboost::pboost].
#' @param maxK Parameter passed to [pboost::pboost].
#' @param verbose Parameter passed to [pboost::pboost].
#' 
#' @return A `lm` model object fitted on the selected features.
#' 
#' @details `plm` is an equivalent implementation to the sequential lasso method
#' proposed by Luo and Chen(2014, \doi{10.1080/01621459.2013.877275}).
#' 
#' @references
#' * Zengchao Xu, Shan Luo and Zehua Chen (2022). Partial profile score feature
#' selection in high-dimensional generalized linear interaction models.
#' Statistics and Its Interface. \doi{10.4310/21-SII706}
#' 
#' * Shan Luo and Zehua Chen (2014). A Sequential Lasso Method for Feature Selection
#' with Ultra-High Dimensional Feature Space. Journal of the American Statistical
#' Association, 109(507):223–232. \doi{10.1080/01621459.2013.877275}
#' 
#' @examples
#' set.seed(2026)
#' n <- 300
#' p <- 200
#' x <- matrix(rnorm(n*p), n)
#' 
#' eta <- drop( x[, 1:3] %*% runif(3, 1.0, 1.5) )
#' y <- rnorm(n, eta, sd=sd(eta))
#' DF <- data.frame(y, x)
#' 
#' plm(y ~ ., DF, verbose=TRUE)
#' plm(y ~ ., DF, stopFun=BIC, verbose=TRUE)
#' pboost(y, x, lm, residuals, verbose=TRUE)
#' 
#' flm(y ~ ., DF, verbose=TRUE)
#' flm(y ~ ., DF, stopFun=BIC, verbose=TRUE)
#' frs(y, x, lm, verbose=TRUE)
#' 
NULL
#> NULL


#' @rdname plm
#' @order 1
#' @export
plm <- function(formula, data, subset, weights, na.action,
                method = "qr", model = TRUE, x = FALSE, y = FALSE, qr = TRUE,
                singular.ok = TRUE, contrasts = NULL, offset, ...,
                stopFun = "EBIC",
                keep = NULL, maxK = NULL, verbose = FALSE) {

    fml <- Formula(formula)
    mf <- model.frame(fml, data=data)

    yvec <- model.part(fml, data=mf, lhs=1, drop=TRUE)
    xmat <- model.part(fml, data=mf, rhs=1) |> as.matrix()
    use.intercept <- attr(terms(mf), "intercept") == 1L


    mc <- match.call(expand.dots = TRUE)
    provided_args <- as.list(mc)[-1]
    provided_args <- provided_args[!(names(provided_args) %in% c("scoreFun"))]
    provided_args$formula <- NULL
    provided_args$data <- NULL

    args <- list(
        fitFun = lm,
        yvec = yvec,
        xmat = xmat,
        use.intercept = use.intercept,
        scoreFun = residuals
    )
    args <- c(args, provided_args)

    return(do.call(pboost, args))
}


#' @rdname plm
#' @order 2
#' @export
plm.fit <- function(x, y, offset = NULL, method = "qr",
                    tol = 1e-07, singular.ok = TRUE, ...,
                    stopFun = "EBIC",
                    keep = NULL, maxK = NULL, verbose = FALSE) {

    n <- NROW(x)
    p <- NCOL(x)
    
    if (is.character(stopFun) && stopFun == "EBIC")
        stopFun <- function(object) {
            class(object) <- "lm"

            ebic.r <- max( 0.0, 1.0 - log(n) / (2.0 * log(p)) )
            stopifnot( ebic.r >= 0 )

            dof <- length(coef(object))
            ebic.penalty <- 2.0 * ebic.r * lchoose(p - length(keep), dof - length(keep))
            stopifnot( is.finite(ebic.penalty) )

            return(BIC(object) + ebic.penalty)
        }
        
    
    mc <- match.call(expand.dots = TRUE)
    provided_args <- as.list(mc)[-1]
    provided_args <- provided_args[!(names(provided_args) %in% c("scoreFun", "stopFun"))]
    provided_args$x <- NULL
    provided_args$y <- NULL

    args <- list(
        fitFun = lm.fit,
        yvec = y,
        xmat = x,
        scoreFun = residuals,
        stopFun = stopFun,
        use.formula = FALSE
    )
    args <- c(args, provided_args)

    return(do.call(pboost, args))
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/EBIC.R
\name{EBIC}
\alias{EBIC}
\title{Extended Bayesian Information Criterion}
\usage{
EBIC(object, p, p.keep, ...)
}
\arguments{
\item{object}{Fitted model object.}

\item{p}{Total number of candidate features, which is available in \link{pboost}.}

\item{p.keep}{Number of features that are pre-specified to be kept in model.}

\item{...}{Additional parameters, which is available in \link{pboost}.}
}
\value{
A function to obtain the EBIC value of a fitted object.
}
\description{
The Extended BIC possesses the selection consistency in
high-dimensional model.

It can be called by the fitted model that has standard \link{logLik} method
to access the attributes \code{nobs} and \code{df}, such as \link{lm}, \link{glm}.
}
\details{
The extended BIC (EBIC) is defined as

\if{html}{\out{<div class="sourceCode">}}\preformatted{EBIC(obj) = BIC(obj) + 2 * r * log(choose(p - |p.keep|, df - |p.keep|)).
}\if{html}{\out{</div>}}
}
\references{
\itemize{
\item Jiahua Chen and Zehua Chen (2008). Extended Bayesian information criteria for
model selection with large model spaces. Biometrika, 95(3):759–771.
\doi{10.1093/biomet/asn034}
\item Jiahua Chen and Zehua Chen (2012). Extended BIC for small-n-large-p sparse GLM.
Statistical Sinica, 22(2):555–574. \doi{10.5705/ss.2010.216}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fsar.R
\name{flagsarlm}
\alias{flagsarlm}
\alias{fsar.fit}
\alias{fsar}
\title{Forward Stepwise Spatial Auto-Regressive Model}
\usage{
flagsarlm(
  formula,
  data = list(),
  listw,
  na.action,
  Durbin,
  type,
  method = "eigen",
  quiet = NULL,
  zero.policy = NULL,
  interval = NULL,
  tol.solve = .Machine$double.eps,
  trs = NULL,
  control = list(),
  selectFun = logLik,
  stopFun = "EBIC",
  keep = NULL,
  maxK = NULL,
  verbose = FALSE
)

fsar.fit(
  x,
  y,
  w,
  selectFun = logLik,
  stopFun = "EBIC",
  keep = NULL,
  maxK = NULL,
  verbose = FALSE
)
}
\arguments{
\item{formula}{Parameters passed to \link[spatialreg:lagsarlm]{spatialreg::lagsarlm}.}

\item{data}{Parameters passed to \link[spatialreg:lagsarlm]{spatialreg::lagsarlm}.}

\item{listw}{Parameters passed to \link[spatialreg:lagsarlm]{spatialreg::lagsarlm}.}

\item{na.action}{Parameters passed to \link[spatialreg:lagsarlm]{spatialreg::lagsarlm}.}

\item{Durbin}{Parameters passed to \link[spatialreg:lagsarlm]{spatialreg::lagsarlm}.}

\item{type}{Parameters passed to \link[spatialreg:lagsarlm]{spatialreg::lagsarlm}.}

\item{method}{Parameters passed to \link[spatialreg:lagsarlm]{spatialreg::lagsarlm}.}

\item{quiet}{Parameters passed to \link[spatialreg:lagsarlm]{spatialreg::lagsarlm}.}

\item{zero.policy}{Parameters passed to \link[spatialreg:lagsarlm]{spatialreg::lagsarlm}.}

\item{interval}{Parameters passed to \link[spatialreg:lagsarlm]{spatialreg::lagsarlm}.}

\item{tol.solve}{Parameters passed to \link[spatialreg:lagsarlm]{spatialreg::lagsarlm}.}

\item{trs}{Parameters passed to \link[spatialreg:lagsarlm]{spatialreg::lagsarlm}.}

\item{control}{Parameters passed to \link[spatialreg:lagsarlm]{spatialreg::lagsarlm}.}

\item{selectFun}{Parameter passed to \link{frs}.}

\item{stopFun}{Parameter passed to \link{frs}.}

\item{keep}{Parameter passed to \link{frs}.}

\item{maxK}{Parameter passed to \link{frs}.}

\item{verbose}{Parameter passed to \link{frs}.}

\item{x}{Matrix of covariates.}

\item{y}{Vector of response.}

\item{w}{Weight matrix (row-sum scaled being one).}
}
\value{
Model object fitted on the selected features.
}
\description{
Forward stepwise strategy for spatial auto-regressive model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/flm.R
\name{flm}
\alias{flm}
\alias{flm.fit}
\title{Forward Regression Selection for Linear Models.}
\usage{
flm(
  formula,
  data,
  subset,
  weights,
  na.action,
  method = "qr",
  model = TRUE,
  x = FALSE,
  y = FALSE,
  qr = TRUE,
  singular.ok = TRUE,
  contrasts = NULL,
  offset,
  ...,
  selectFun = logLik,
  stopFun = "EBIC",
  keep = NULL,
  maxK = NULL,
  verbose = FALSE
)

flm.fit(
  x,
  y,
  offset = NULL,
  method = "qr",
  tol = 1e-07,
  singular.ok = TRUE,
  ...,
  selectFun = "logLik",
  stopFun = "EBIC",
  keep = NULL,
  maxK = NULL,
  verbose = FALSE
)
}
\arguments{
\item{formula}{Parameter passed to \link{lm}.}

\item{data}{Parameter passed to \link{lm}.}

\item{subset}{Parameter passed to \link{lm}.}

\item{weights}{Parameter passed to \link{lm}.}

\item{na.action}{Parameter passed to \link{lm}.}

\item{method}{Parameter passed to \link{lm}.}

\item{model}{Parameter passed to \link{lm}.}

\item{x}{Parameter passed to \link{lm}.}

\item{y}{Parameter passed to \link{lm}.}

\item{qr}{Parameter passed to \link{lm}.}

\item{singular.ok}{Parameter passed to \link{lm}.}

\item{contrasts}{Parameter passed to \link{lm}.}

\item{offset}{Parameter passed to \link{lm} or \link{lm.fit}.}

\item{...}{Parameters passed to \link{lm} or \link{lm.fit}.}

\item{selectFun}{Parameter passed to \link{frs}.}

\item{stopFun}{Parameter passed to \link{frs}.}

\item{keep}{Parameter passed to \link{frs}.}

\item{maxK}{Parameter passed to \link{frs}.}

\item{verbose}{Parameter passed to \link{frs}.}

\item{tol}{Parameter passed to \link{lm.fit}.}
}
\value{
A \code{lm} model object fitted on the selected features.
}
\description{
\code{flm()} inherits the usage of the function \link{lm}, and performs forward regression selection for linear models.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/penalty-glmnet.R
\name{coef.penaltyglmnet}
\alias{coef.penaltyglmnet}
\title{Extract Coefficients from Lasso Penalized Models}
\usage{
\method{coef}{penaltyglmnet}(object, ...)
}
\arguments{
\item{object}{Object.}

\item{...}{Additional arguments (not used).}
}
\value{
Named vector of non-zero coefficients of under cross-validation error.
}
\description{
Extract Coefficients from Lasso Penalized Models
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pboost.R
\name{pboost}
\alias{pboost}
\title{Profile Boosting Framework}
\usage{
pboost(
  yvec,
  xmat,
  fitFun,
  scoreFun,
  stopFun = "EBIC",
  ...,
  use.formula = TRUE,
  use.intercept = TRUE,
  keep = NULL,
  maxK = NULL,
  verbose = FALSE
)
}
\arguments{
\item{yvec}{Response vector.}

\item{xmat}{Numeric feature matrix.}

\item{fitFun}{Function to fit the empirical risk function in
the form \code{fitFun(formula, data, ...)}.}

\item{scoreFun}{Function to compute the derivative, denoted by \eqn{\frac{\partial \ell(y, \eta)}{\partial \eta}}, of empirical risk function in the form \code{scoreFun(object)}, where \code{object} is returned by \code{fitFun}.
\code{scoreFun()} should return a vector with the same length of \code{y} in \code{data}.}

\item{stopFun}{Stopping rule for profile boosting, which has the form
\code{stopFun(object)} to evaluate the performance of model \code{object} returned
by \code{fitFun}, such as \link{EBIC} or \link{BIC}.}

\item{...}{Additional arguments to be passed to \code{fitFun}.}

\item{use.formula}{Whether to use formula interface for model fitting. Default to \code{TRUE}.
When \code{use.formula=TRUE}, the the model fitting function has the form \code{fitFun(formula, data, ...)}; otherwise, \code{fitFun(x, y, ...)}.}

\item{use.intercept}{Include intercept in the model fitting? Valid only when \code{use.formula=TRUE}.}

\item{keep}{Vector of indices or feature names initial features to include.}

\item{maxK}{Maximal number of identified features.
If \code{maxK} is specified, it will suppress \code{stopFun}, saying that the
profile boosting continues until the procedure identifies \code{maxK} features.
The pre-specified features in \code{keep} are counted toward \code{maxK}.}

\item{verbose}{Print the procedure path?}
}
\value{
Model object fitted on the selected features.
}
\description{
\code{pboost()} is the generic workhorse function of profile boosting
framework for parametric regression.
}
\seealso{
\link{pbetareg}, \link{pcoxph}, \link{pglm}, \link{plm}, \link{prq}, \link{psar}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/frs.R
\name{frs}
\alias{frs}
\title{Forward Regression Selection Framework}
\usage{
frs(
  yvec,
  xmat,
  fitFun,
  ...,
  use.formula = TRUE,
  use.intercept = TRUE,
  selectFun = logLik,
  stopFun = "EBIC",
  keep = NULL,
  maxK = NULL,
  verbose = FALSE
)
}
\arguments{
\item{yvec}{See \link{pboost}.}

\item{xmat}{See \link{pboost}.}

\item{fitFun}{See \link{pboost}.}

\item{...}{See \link{pboost}.}

\item{use.formula}{See \link{pboost}.}

\item{use.intercept}{See \link{pboost}.}

\item{selectFun}{A function to evaluate the importance of an unselected feature when it is added to current model.
The default is \code{logLik}, meaning that the feature with the largest post-added log-likelihood is identified as the next one to be added to the model.
Note that \code{selectFun} is only used for selecting features, and it does not affect the stopping rule of forward regression selection, which is determined by \code{stopFun}.}

\item{stopFun}{See \link{pboost}.}

\item{keep}{See \link{pboost}.}

\item{maxK}{See \link{pboost}.}

\item{verbose}{See \link{pboost}.}
}
\value{
Model object fitted on the selected features.
}
\description{
\code{frs()} is a generic workhorse function of forward regression selection for parametric regression.
}
\seealso{
\link{fbetareg}, \link{fcoxph}, \link{fglm}, \link{flm}, \link{frq}, \link{fsar}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/penalty-rq.R
\name{coef.penaltyrq}
\alias{coef.penaltyrq}
\title{Extract Coefficients from Non-Convex Penalized Models}
\usage{
\method{coef}{penaltyrq}(object, ...)
}
\arguments{
\item{object}{Object.}

\item{...}{Additional arguments (not used).}
}
\value{
A named vector of coefficients.
}
\description{
Extract Coefficients from Non-Convex Penalized Models
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sar.R
\name{sar.model}
\alias{sar.model}
\alias{get_rho}
\alias{sar.fit}
\alias{residuals.sarpboost}
\alias{logLik.sarpboost}
\alias{BIC.sarpboost}
\alias{coef.sarpboost}
\alias{set_rook_matrix}
\title{Functions for Spatial Auto-regressive Model}
\usage{
get_rho(x, y, w)

sar.fit(x, y, w)

\method{residuals}{sarpboost}(object, ...)

\method{logLik}{sarpboost}(object, ...)

\method{BIC}{sarpboost}(object, ...)

\method{coef}{sarpboost}(object, ...)

set_rook_matrix(rNum, cNum)
}
\arguments{
\item{x}{Numeric feature matrix.}

\item{y}{Response vector.}

\item{w}{Weight matrix (row-sum scaled).}

\item{object}{Object.}

\item{...}{Arguments.}

\item{rNum}{Number of row units in Rook structure.}

\item{cNum}{Number of column units in Rook structure.}
}
\value{
Value or list.
}
\description{
\itemize{
\item \code{get_rho()}: MLE of \eqn{\rho} in SAR model. See \link{psar}.
\item \code{sar.fit()}: Fit SAR model with given \eqn{\rho}.
\item \code{set_rook_matrix()}: Construct Rook weight adjacency matrix.
\item \code{logLik()}, \code{BIC()}, \code{residuals()}, \code{coef()}.
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/frq.R
\name{frq}
\alias{frq}
\title{Forward Regression Selection for Quantile Regression Models}
\usage{
frq(
  formula,
  tau = 0.5,
  data,
  subset,
  weights,
  na.action,
  method = "br",
  model = TRUE,
  contrasts = NULL,
  ...,
  selectFun = logLik,
  stopFun = "EBIC",
  keep = NULL,
  maxK = NULL,
  verbose = FALSE
)
}
\arguments{
\item{formula}{Parameter passed to \link[quantreg:rq]{quantreg::rq}.}

\item{tau}{Parameter passed to \link[quantreg:rq]{quantreg::rq}.}

\item{data}{Parameter passed to \link[quantreg:rq]{quantreg::rq}.}

\item{subset}{Parameter passed to \link[quantreg:rq]{quantreg::rq}.}

\item{weights}{Parameter passed to \link[quantreg:rq]{quantreg::rq}.}

\item{na.action}{Parameter passed to \link[quantreg:rq]{quantreg::rq}.}

\item{method}{Parameter passed to \link[quantreg:rq]{quantreg::rq}.}

\item{model}{Parameter passed to \link[quantreg:rq]{quantreg::rq}.}

\item{contrasts}{Parameter passed to \link[quantreg:rq]{quantreg::rq}.}

\item{...}{Parameters passed to \link[quantreg:rq]{quantreg::rq}.}

\item{selectFun}{Parameter passed to \link{frs}.}

\item{stopFun}{Parameter passed to \link{frs}.}

\item{keep}{Parameter passed to \link{frs}.}

\item{maxK}{Parameter passed to \link{frs}.}

\item{verbose}{Parameter passed to \link{frs}.}
}
\description{
\code{frq()} inherits the usage of the function \link[quantreg:rq]{quantreg::rq}, and performs forward regression selection for quantile regression models.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/plm.R
\name{plm}
\alias{plm}
\alias{plm.fit}
\title{Profile Boosting for Linear Models.}
\usage{
plm(
  formula,
  data,
  subset,
  weights,
  na.action,
  method = "qr",
  model = TRUE,
  x = FALSE,
  y = FALSE,
  qr = TRUE,
  singular.ok = TRUE,
  contrasts = NULL,
  offset,
  ...,
  stopFun = "EBIC",
  keep = NULL,
  maxK = NULL,
  verbose = FALSE
)

plm.fit(
  x,
  y,
  offset = NULL,
  method = "qr",
  tol = 1e-07,
  singular.ok = TRUE,
  ...,
  stopFun = "EBIC",
  keep = NULL,
  maxK = NULL,
  verbose = FALSE
)
}
\arguments{
\item{formula}{Parameter passed to \link{lm}.}

\item{data}{Parameter passed to \link{lm}.}

\item{subset}{Parameter passed to \link{lm}.}

\item{weights}{Parameter passed to \link{lm}.}

\item{na.action}{Parameter passed to \link{lm}.}

\item{method}{Parameter passed to \link{lm}.}

\item{model}{Parameter passed to \link{lm}.}

\item{x}{Parameter passed to \link{lm}.}

\item{y}{Parameter passed to \link{lm}.}

\item{qr}{Parameter passed to \link{lm}.}

\item{singular.ok}{Parameter passed to \link{lm}.}

\item{contrasts}{Parameter passed to \link{lm}.}

\item{offset}{Parameter passed to \link{lm} or \link{lm.fit}.}

\item{...}{Parameters passed to \link{lm} or \link{lm.fit}.}

\item{stopFun}{Parameter passed to \link{pboost}.}

\item{keep}{Parameter passed to \link{pboost}.}

\item{maxK}{Parameter passed to \link{pboost}.}

\item{verbose}{Parameter passed to \link{pboost}.}

\item{tol}{Parameter passed to \link{lm.fit}.}
}
\value{
A \code{lm} model object fitted on the selected features.
}
\description{
\link{plm} inherits the usage of the built-in function \link{lm}.
}
\details{
\code{plm} is an equivalent implementation to the sequential lasso method
proposed by Luo and Chen(2014, \doi{10.1080/01621459.2013.877275}).
}
\examples{
set.seed(2026)
n <- 300
p <- 200
x <- matrix(rnorm(n*p), n)

eta <- drop( x[, 1:3] \%*\% runif(3, 1.0, 1.5) )
y <- rnorm(n, eta, sd=sd(eta))
DF <- data.frame(y, x)

plm(y ~ ., DF, verbose=TRUE)
plm(y ~ ., DF, stopFun=BIC, verbose=TRUE)
pboost(y, x, lm, residuals, verbose=TRUE)

flm(y ~ ., DF, verbose=TRUE)
flm(y ~ ., DF, stopFun=BIC, verbose=TRUE)
frs(y, x, lm, verbose=TRUE)

}
\references{
\itemize{
\item Zengchao Xu, Shan Luo and Zehua Chen (2022). Partial profile score feature
selection in high-dimensional generalized linear interaction models.
Statistics and Its Interface. \doi{10.4310/21-SII706}
\item Shan Luo and Zehua Chen (2014). A Sequential Lasso Method for Feature Selection
with Ultra-High Dimensional Feature Space. Journal of the American Statistical
Association, 109(507):223–232. \doi{10.1080/01621459.2013.877275}
}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fcoxph.R
\name{fcoxph}
\alias{fcoxph}
\title{Forward Regression Selection for Cox proportional hazards Model}
\usage{
fcoxph(
  formula,
  data,
  weights,
  subset,
  na.action,
  init,
  control,
  ties = c("efron", "breslow", "exact"),
  singular.ok = TRUE,
  robust,
  model = FALSE,
  x = FALSE,
  y = TRUE,
  tt,
  method = ties,
  id,
  cluster,
  istate,
  statedata,
  nocenter = c(-1, 0, 1),
  ...,
  selectFun = logLik,
  stopFun = "EBIC",
  keep = NULL,
  maxK = NULL,
  verbose = FALSE
)
}
\arguments{
\item{formula}{Parameter passed to \link[survival:coxph]{survival::coxph}.}

\item{data}{Parameter passed to \link[survival:coxph]{survival::coxph}.}

\item{weights}{Parameter passed to \link[survival:coxph]{survival::coxph}.}

\item{subset}{Parameter passed to \link[survival:coxph]{survival::coxph}.}

\item{na.action}{Parameter passed to \link[survival:coxph]{survival::coxph}.}

\item{init}{Parameter passed to \link[survival:coxph]{survival::coxph}.}

\item{control}{Parameter passed to \link[survival:coxph]{survival::coxph}.}

\item{ties}{Parameter passed to \link[survival:coxph]{survival::coxph}.}

\item{singular.ok}{Parameter passed to \link[survival:coxph]{survival::coxph}.}

\item{robust}{Parameter passed to \link[survival:coxph]{survival::coxph}.}

\item{model}{Parameter passed to \link[survival:coxph]{survival::coxph}.}

\item{x}{Parameter passed to \link[survival:coxph]{survival::coxph}.}

\item{y}{Parameter passed to \link[survival:coxph]{survival::coxph}.}

\item{tt}{Parameter passed to \link[survival:coxph]{survival::coxph}.}

\item{method}{Parameter passed to \link[survival:coxph]{survival::coxph}.}

\item{id}{Parameter passed to \link[survival:coxph]{survival::coxph}.}

\item{cluster}{Parameter passed to \link[survival:coxph]{survival::coxph}.}

\item{istate}{Parameter passed to \link[survival:coxph]{survival::coxph}.}

\item{statedata}{Parameter passed to \link[survival:coxph]{survival::coxph}.}

\item{nocenter}{Parameter passed to \link[survival:coxph]{survival::coxph}.}

\item{...}{Parameters passed to \link[survival:coxph]{survival::coxph}.}

\item{selectFun}{Parameter passed to \link{frs}.}

\item{stopFun}{Parameter passed to \link{frs}.}

\item{keep}{Parameter passed to \link{frs}.}

\item{maxK}{Parameter passed to \link{frs}.}

\item{verbose}{Parameter passed to \link{frs}.}
}
\value{
A \code{coxph} model object fitted on the selected features.
}
\description{
Forward regression selection for Cox model.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pggm.R
\name{pggm}
\alias{pggm}
\title{Profile Boosting for Gaussian Graphical Model}
\usage{
pggm(
  S,
  nObs,
  maxK = floor(min(nObs - 1, NROW(S) - 1, 50)),
  digits = 8,
  verbose = FALSE
)
}
\arguments{
\item{S}{Covariance matrix.}

\item{nObs}{Number of observations.}

\item{maxK}{Maximum number of identified edges.}

\item{digits}{Integer indicating the number of decimal
places or significant digits to be used.}

\item{verbose}{Print the procedure path?}
}
\value{
Index set of identified features.
}
\description{
Profile boosting for Gaussian graphical model.
}
\examples{
library(MASS)
library(Matrix)

set.seed(2025)
n <- 1000
p <- 10

Omega <- Diagonal(p)
diag(Omega[1:4, 2:5]) <- diag(Omega[2:5, 1:4]) <- 0.5
Sigma <- chol2inv(chol(Omega))
X <- mvrnorm(n, rep(0, p), Sigma, empirical=TRUE)
S <- cov(X)
system.time( egg <- pggm(S, n) )

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pglm.R
\name{pglm}
\alias{pglm}
\alias{pglm.fit}
\title{Profile Boosting for Generalized Linear Models.}
\usage{
pglm(
  formula,
  family = gaussian,
  data,
  weights,
  subset,
  na.action,
  start = NULL,
  etastart,
  mustart,
  offset,
  control = list(...),
  model = TRUE,
  method = "glm.fit",
  x = FALSE,
  y = TRUE,
  singular.ok = TRUE,
  contrasts = NULL,
  ...,
  stopFun = "EBIC",
  keep = NULL,
  maxK = NULL,
  verbose = FALSE
)

pglm.fit(
  x,
  y,
  weights = rep.int(1, NROW(y)),
  start = NULL,
  etastart = NULL,
  mustart = NULL,
  offset = rep.int(0, NROW(y)),
  family = gaussian(),
  control = list(),
  intercept = TRUE,
  singular.ok = TRUE,
  stopFun = "EBIC",
  keep = NULL,
  maxK = NULL,
  verbose = FALSE
)
}
\arguments{
\item{formula}{Parameter passed to \link{glm}.}

\item{family}{Parameter passed to \link{glm}.}

\item{data}{Parameter passed to \link{glm}.}

\item{weights}{Parameter passed to \link{glm}.}

\item{subset}{Parameter passed to \link{glm}.}

\item{na.action}{Parameter passed to \link{glm}.}

\item{start}{Parameter passed to \link{glm}.}

\item{etastart}{Parameter passed to \link{glm}.}

\item{mustart}{Parameter passed to \link{glm}.}

\item{offset}{Parameter passed to \link{glm}.}

\item{control}{Parameter passed to \link{glm}.}

\item{model}{Parameter passed to \link{glm}.}

\item{method}{Parameter passed to \link{glm}.}

\item{x}{Parameter passed to \link{glm}.}

\item{y}{Parameter passed to \link{glm}.}

\item{singular.ok}{Parameter passed to \link{glm}.}

\item{contrasts}{Parameter passed to \link{glm}.}

\item{...}{Parameters passed to \link{glm}.}

\item{stopFun}{Parameter passed to \link{pboost}.}

\item{keep}{Parameter passed to \link{pboost}.}

\item{maxK}{Parameter passed to \link{pboost}.}

\item{verbose}{Parameter passed to \link{pboost}.}

\item{intercept}{Parameter passed to \link{glm.fit}.}
}
\value{
A \code{glm} model object fitted on the selected features.
}
\description{
\link{pglm} inherits the usage of the built-in function \link{glm}.
}
\examples{
set.seed(2026)
n <- 200
p <- 100
x <- matrix(rnorm(n*p), n)

eta <- drop( x[, 1:3] \%*\% runif(3, 1.0, 1.5) )
y <- rbinom(n, 1, 1/(1+exp(-eta)))
DF <- data.frame(y, x)

## ---------- pboost ----------
pglm(y ~ ., "binomial", DF, verbose=TRUE)
pglm(y ~ ., "binomial", DF, stopFun=BIC, verbose=TRUE)

scoreLogistic <- function(object) {
   eta.hat <- object[["linear.predictors"]]
   return(object[["y"]] - 1/(1+exp(-eta.hat)))
}
(result <- pboost(y, x, glm, scoreLogistic, family="binomial", verbose=TRUE))
all.vars(formula(result)[[3]])

## ---------- frs ----------
fglm(y ~ ., "binomial", DF, verbose=TRUE)
fglm(y ~ ., "binomial", DF, stopFun=BIC, verbose=TRUE)

frs(y, x, glm, family="binomial", verbose=TRUE)

}
\references{
Zengchao Xu, Shan Luo and Zehua Chen (2022). Partial profile score feature selection
in high-dimensional generalized linear interaction models. Statistics and Its Interface.
\doi{10.4310/21-SII706}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/psar.R
\name{psar}
\alias{psar}
\title{Profile Boosting for Spatial Auto-regressive Model}
\usage{
psar(x, y, w, stopFun = "EBIC", keep = NULL, maxK = NULL, verbose = FALSE)
}
\arguments{
\item{x}{Numeric feature matrix.}

\item{y}{Response vector.}

\item{w}{Weight matrix (row-sum scaled being one).}

\item{stopFun}{Parameter passed to \link{pboost}.}

\item{keep}{Parameter passed to \link{pboost}.}

\item{maxK}{Parameter passed to \link{pboost}.}

\item{verbose}{Parameter passed to \link{pboost}.}
}
\value{
Model object fitted on the selected features.
}
\description{
Profile boosting variable selection for spatial auto-regressive (SAR) model
\deqn{\bm{y} = \rho W \bm{y} + X \beta + \bm{\varepsilon},}
where \eqn{\bm{y}, \bm{\varepsilon} \in \mathbb{R}^n}, \eqn{X \in \mathbb{R}^{n \times p}},
\eqn{\rho \in (-1, 1)}.

\code{psar()} implements the PVS procedure for data matrix.
}
\examples{
set.seed(2026)

w <- set_rook_matrix(9, 9)

n <- NROW(w)
p <- 300
x <- matrix(rnorm(n*p), n) \%*\% chol(0.7^abs(outer(1:p, 1:p, "-")))
eta <- drop(x[, 1:3] \%*\% runif(3, 1.5, 2.0))

rho0 <- 0.2
sig0 <- 1.0
y <- solve(diag(n) - rho0 * w, rnorm(n, eta, sd=sig0)) |> drop()

## ---------- pboost ----------
system.time( egg <- psar(x, y, w, verbose=TRUE) )
y.tilde <- (diag(NROW(x)) - egg[["rho"]] * w) \%*\% y

beta.hat <- egg[["beta"]]
idx <- as.integer(sub("[[:alpha:]]", "", names(beta.hat)))
sig2.hat <- mean( (y.tilde - drop(x[, idx, drop=FALSE] \%*\% beta.hat))^2 )
print( egg[["sig2"]] - sig2.hat )

## ---------- frs ----------
fsar(x, y, w, verbose=TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pbetareg.R
\name{pbetareg}
\alias{pbetareg}
\title{Profile Boosting for Beta Regression}
\usage{
pbetareg(
  formula,
  data,
  subset,
  na.action,
  weights,
  offset,
  link = c("logit", "probit", "cloglog", "cauchit", "log", "loglog"),
  link.phi = NULL,
  type = c("ML", "BC", "BR"),
  dist = NULL,
  nu = NULL,
  control = betareg.control(...),
  model = TRUE,
  y = TRUE,
  x = FALSE,
  ...,
  stopFun = "EBIC",
  keep = NULL,
  maxK = NULL,
  verbose = FALSE
)
}
\arguments{
\item{formula}{Parameter passed to \link[betareg:betareg]{betareg::betareg}.}

\item{data}{Parameter passed to \link[betareg:betareg]{betareg::betareg}.}

\item{subset}{Parameter passed to \link[betareg:betareg]{betareg::betareg}.}

\item{na.action}{Parameter passed to \link[betareg:betareg]{betareg::betareg}.}

\item{weights}{Parameter passed to \link[betareg:betareg]{betareg::betareg}.}

\item{offset}{Parameter passed to \link[betareg:betareg]{betareg::betareg}.}

\item{link}{Parameter passed to \link[betareg:betareg]{betareg::betareg}.}

\item{link.phi}{Parameter passed to \link[betareg:betareg]{betareg::betareg}.}

\item{type}{Parameter passed to \link[betareg:betareg]{betareg::betareg}.}

\item{dist}{Parameter passed to \link[betareg:betareg]{betareg::betareg}.}

\item{nu}{Parameter passed to \link[betareg:betareg]{betareg::betareg}.}

\item{control}{Parameter passed to \link[betareg:betareg]{betareg::betareg}.}

\item{model}{Parameter passed to \link[betareg:betareg]{betareg::betareg}.}

\item{y}{Parameter passed to \link[betareg:betareg]{betareg::betareg}.}

\item{x}{Parameter passed to \link[betareg:betareg]{betareg::betareg}.}

\item{...}{Parameters passed to \link[betareg:betareg]{betareg::betareg}.}

\item{stopFun}{Parameter passed to \link{pboost}.}

\item{keep}{Parameter passed to \link{pboost}.}

\item{maxK}{Parameter passed to \link{pboost}.}

\item{verbose}{Parameter passed to \link{pboost}.}
}
\value{
A \code{betareg} model object fitted on the selected features.
}
\description{
\link{pbetareg} inherits the usage of \link[betareg:betareg]{betareg::betareg}.
}
\examples{
\dontrun{
set.seed(2026)
n <- 300
p <- 20
x <- matrix(runif(n*p), n)
mu <- runif(n)
phi <- 1.0

shape1 <- mu * phi
shape2 <- (1-mu) * phi
y <- rbeta(n, shape1, shape2)
DF <- data.frame(y, x)

pbetareg(y ~ ., DF, verbose=TRUE)
fbetareg(y ~ ., DF, verbose=TRUE)
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/prq.R
\name{prq}
\alias{prq}
\title{Profile Boosting for Quantile Regression Models}
\usage{
prq(
  formula,
  tau = 0.5,
  data,
  subset,
  weights,
  na.action,
  method = "br",
  model = TRUE,
  contrasts = NULL,
  ...,
  stopFun = "EBIC",
  keep = NULL,
  maxK = NULL,
  verbose = FALSE
)
}
\arguments{
\item{formula}{Parameter passed to \link[quantreg:rq]{quantreg::rq}.}

\item{tau}{Parameter passed to \link[quantreg:rq]{quantreg::rq}.}

\item{data}{Parameter passed to \link[quantreg:rq]{quantreg::rq}.}

\item{subset}{Parameter passed to \link[quantreg:rq]{quantreg::rq}.}

\item{weights}{Parameter passed to \link[quantreg:rq]{quantreg::rq}.}

\item{na.action}{Parameter passed to \link[quantreg:rq]{quantreg::rq}.}

\item{method}{Parameter passed to \link[quantreg:rq]{quantreg::rq}.}

\item{model}{Parameter passed to \link[quantreg:rq]{quantreg::rq}.}

\item{contrasts}{Parameter passed to \link[quantreg:rq]{quantreg::rq}.}

\item{...}{Parameters passed to \link[quantreg:rq]{quantreg::rq}.}

\item{stopFun}{Parameter passed to \link{pboost}.}

\item{keep}{Parameter passed to \link{pboost}.}

\item{maxK}{Parameter passed to \link{pboost}.}

\item{verbose}{Parameter passed to \link{pboost}.}
}
\value{
A \code{rq} model object fitted on the selected features.
}
\description{
\link{prq} inherits the usage of the function \link[quantreg:rq]{quantreg::rq}.
}
\examples{
library(quantreg)
set.seed(2026)
n <- 300
p <- 20
x <- matrix(rnorm(n*p), n)

eta <- drop( x[, 1:3] \%*\% runif(3, 1.0, 1.5) )
y <- eta + (1.0 + x[, 3]) * rnorm(n)
DF <- data.frame(y, x)

tau <- 0.5
prq(y ~ ., tau, DF, verbose=TRUE)

BIC <- function(obj) AIC(obj, k=-1)
prq(y ~ ., tau, DF, stopFun=BIC, verbose=TRUE)
frq(y ~ ., tau, DF, stopFun=BIC, verbose=TRUE)

scoreFun <- function(object)
   return(ifelse(object[["y"]] < fitted(object), tau - 1, tau))

pboost(y, x, rq, scoreFun, BIC, tau=tau, verbose=TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/penalization.R, R/penalty-glmnet.R,
%   R/penalty-ncvreg.R, R/penalty-rq.R, R/penalty-sar.R
\name{penalization}
\alias{penalization}
\alias{pen_glmnet}
\alias{pen_ncvreg}
\alias{pen_rq}
\alias{pen_sar}
\title{Penalized Methods for Feature Selection}
\usage{
pen_glmnet(x, y, family)

pen_ncvreg(x, y, family, penalty)

pen_rq(x, y, tau, penalty)

pen_sar(x, y, rho, w, penalty)
}
\arguments{
\item{x}{Feature matrix}

\item{y}{Response vector.}

\item{family}{"gaussian", "binomial", or "cox".}

\item{penalty}{"lasso", "SCAD" or "MCP".}

\item{tau}{Quantiles to be modeled.}

\item{rho}{Spatial autoregressive parameter. If missing or NULL, it will be estimated.}

\item{w}{Weight matrix (row-sum scaled being one).}
}
\value{
Set of selected features.
}
\description{
Penalized methods for feature selection.
These functions are only designed for comparison of numerical simulations.

\itemize{
\item \code{pen_glmnet(x, y, family)}: LASSO penalized models.
}

\itemize{
\item \code{pen_ncvreg(x, y, family, penalty)}: Non-Convex penalized models.
}

\itemize{
\item \code{pen_rq(x, y, tau, penalty)}: penalized quantile regression models.
}

\itemize{
\item \code{pen_sar(x, y, rho, w, penalty)}: penalized spatial auto-regressive models.
}
}
\examples{
library(survival)
set.seed(2026)
n <- 10
p <- 20

x <- replicate(p, rnorm(n))
b0 <- runif(3, 1.5, 2.0)
eta <- drop(x[, 1:3] \%*\% b0)

## ---------- linear ----------
y <- rnorm(n, eta)

pen_glmnet(x=x, y=y, family="gaussian") |> coef()
pen_ncvreg(x=x, y=y, family="gaussian", penalty="MCP") |> coef()
pen_ncvreg(x=x, y=y, family="gaussian", penalty="SCAD") |> coef()


## ---------- logistic ----------
y <- rbinom(n, 1, 1.0 / (1.0 + exp(-eta)))

pen_glmnet(x=x, y=y, family="binomial") |> coef()
pen_ncvreg(x=x, y=y, family="binomial", penalty="MCP") |> coef()
pen_ncvreg(x=x, y=y, family="binomial", penalty="SCAD") |> coef()


## ---------- cox ----------
h0 <- 0.01
censoringRate <- 0.3
survivalTime <- -log(runif(n)) / (h0 * exp(eta))
censoringTime <- rexp(n, rate = -log(1 - censoringRate)/median(survivalTime))
y <- cbind(
    time = pmin(survivalTime, censoringTime),
    status = as.numeric(survivalTime <= censoringTime)
)

pen_glmnet(x=x, y=y, family="cox") |> coef()
pen_ncvreg(x=x, y=y, family="cox", penalty="MCP") |> coef()
pen_ncvreg(x=x, y=y, family="cox", penalty="SCAD") |> coef()


## ---------- quantile ----------
y <- eta + rt(n, 2)

pen_rq(x=x, y=y, tau=0.5, penalty="lasso") |> coef()
pen_rq(x=x, y=y, tau=0.5, penalty="MCP") |> coef()
pen_rq(x=x, y=y, tau=0.5, penalty="SCAD") |> coef()


## ---------- sar ----------
w0 <- set_rook_matrix(5, n/5)
rho0 <- 0.5
y <- solve(diag(n) - rho0 * w0, rnorm(n, eta))

pen_sar(x=x, y=y, rho=rho0, w=w0, penalty="lasso") |> coef()
pen_sar(x=x, y=y, rho=rho0, w=w0, penalty="MCP") |> coef()
pen_sar(x=x, y=y, rho=rho0, w=w0, penalty="SCAD") |> coef()

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/pboost-package.R
\docType{package}
\name{pboost-package}
\alias{pboost-package}
\title{pboost: Profile Boosting Framework for Parametric Models}
\description{
A profile boosting framework for feature selection in parametric models. It offers a unified interface pboost() and several wrapped models, including linear model, generalized linear models, quantile regression, Cox proportional hazards model, beta regression, spatial auto-regressive models.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/paradoxical-rhapsody/pboost}
  \item Report bugs at \url{https://github.com/paradoxical-rhapsody/pboost/issues}
}

}
\author{
\strong{Maintainer}: Zengchao Xu \email{zengc.xu@aliyun.com} [copyright holder]

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/fglm.R
\name{fglm}
\alias{fglm}
\alias{fglm.fit}
\title{Forward Regression Selection for Generalized Linear Models}
\usage{
fglm(
  formula,
  family = gaussian,
  data,
  weights,
  subset,
  na.action,
  start = NULL,
  etastart,
  mustart,
  offset,
  control = list(...),
  model = TRUE,
  method = "glm.fit",
  x = FALSE,
  y = TRUE,
  singular.ok = TRUE,
  contrasts = NULL,
  ...,
  selectFun = logLik,
  stopFun = "EBIC",
  keep = NULL,
  maxK = NULL,
  verbose = FALSE
)

fglm.fit(
  x,
  y,
  weights = rep.int(1, NROW(y)),
  start = NULL,
  etastart = NULL,
  mustart = NULL,
  offset = rep.int(0, NROW(y)),
  family = gaussian(),
  control = list(),
  intercept = TRUE,
  singular.ok = TRUE,
  selectFun = "logLik",
  stopFun = "EBIC",
  keep = NULL,
  maxK = NULL,
  verbose = FALSE
)
}
\arguments{
\item{formula}{Parameter passed to \link{glm}.}

\item{family}{Parameter passed to \link{glm}.}

\item{data}{Parameter passed to \link{glm}.}

\item{weights}{Parameter passed to \link{glm}.}

\item{subset}{Parameter passed to \link{glm}.}

\item{na.action}{Parameter passed to \link{glm}.}

\item{start}{Parameter passed to \link{glm}.}

\item{etastart}{Parameter passed to \link{glm}.}

\item{mustart}{Parameter passed to \link{glm}.}

\item{offset}{Parameter passed to \link{glm}.}

\item{control}{Parameter passed to \link{glm}.}

\item{model}{Parameter passed to \link{glm}.}

\item{method}{Parameter passed to \link{glm}.}

\item{x}{Parameter passed to \link{glm}.}

\item{y}{Parameter passed to \link{glm}.}

\item{singular.ok}{Parameter passed to \link{glm}.}

\item{contrasts}{Parameter passed to \link{glm}.}

\item{...}{Parameters passed to \link{glm}.}

\item{selectFun}{Parameter passed to \link{frs}.}

\item{stopFun}{Parameter passed to \link{frs}.}

\item{keep}{Parameter passed to \link{frs}.}

\item{maxK}{Parameter passed to \link{frs}.}

\item{verbose}{Parameter passed to \link{frs}.}

\item{intercept}{Parameter passed to \link{glm.fit}.}
}
\value{
A \code{glm} model object fitted on the selected features.
}
\description{
\code{fglm()} inherits the usage of \link{glm}, and performs forward regression selection for generalized linear models.
}
